package sach;

import sach.file.*;
import sach.gui.*;
import sach.logic.*;
import sach.network.*;

/**
 * Program Šach, main-class
 * @author Adam Netočný
 */
public class SachApp implements ComponentInterface {
    /**
     * Statický odkaz na objekt, ktorý spustil aplikáciu
     */
    public static SachApp me;

    /**
     * Status hry, nesie základné informácie o aktuálnej hre
     */
    public static GameStatus gameStatus;

    /**
     * Detekuje, či sa jedná o testovanie hry alebo nie
     */
    public static boolean test = false;

    /**
     * Odkaz na herný Engine hry
     */
    private Sachovnica logic;
    
    /**
     * Sieťové rozhrannie hry
     */
    private NetworkGame network;
    
    /**
     * Súborové rozhrannie hry
     */
    private ZapisTahov file;

    /**
     * Obsahuje odkaz na hlavné okno aplikácie
     */
    private MainFrame gui;

    /**
     * Štandardizovaný konštruktor, pri vytvorení tohto objektu je vytvorené taktiež hlavné okno programu
     */
    public SachApp() {
        logic = new Sachovnica();
        gameStatus = new GameStatus(Figurka.Color.biela, logic);
        gui = new MainFrame();
        file = new ZapisTahov();

        gui.addTahListener(logic);
        file.addTahListener(logic);
        logic.addTahListener(gui.getTahListener());
        logic.addTahListener(file);
    }

    /**
     * Volá exit() na vytvorených komponentách, program sa volaním tejto meódy ukončí
     */
    public void exit(){
        file.exit();
        logic.exit();
        if(network!=null){
            network.exit();
        }
        gui.exit();

        System.exit(0);
    }

    /**
     * Pripája sieťové rozhrannie k ostaným komponentám
     * @param network Sieťové rozhrannie
     */
    public void setNetwork(NetworkGame network) {
        if(network != null) {
            logic.removeTahListener(this.network);
            this.network = network;
            logic.addTahListener(this.network);
            this.network.addTahListener(logic);
        }
    }

    /**
     * Vytvorí a otvorí hlavné okno programu
     * @param args Parametre príkazového riadku
     */
    public static void main(String[] args) {
        me = new SachApp();
    }
}
