/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package sach.eventmodel;

import java.util.*;
import sach.logic.*;

/**
 * Trieda na obsluhu událostí
 * @author Adam Netočný
 */
public abstract class TahHandler implements TahListener {
    /**
     * Vector obsahujúci všetky listenery pripojené na daný objekt
     */
    private transient Vector listeners;

    /**
     * Posledná odoslaná udalosť
     */
    protected TahEvent event = null;

    /**
     * Volaná pri prijatí události typu ťah
     * @param event Daná udalosť
     */
    public abstract void tah(TahEvent event);

    /**
     * Volaná pri prijatí události typu chybný ťah
     * @param event Daná udalosť
     */
    public abstract void chybyTah(TahEvent event);

    /**
     * Volaná pri prijatí události typu výber figúrky
     * @param event Daná udalosť
     */
    public abstract void vyberFigurku(TahEvent event);

    /**
     * Volaná pri prijatí události typu chybová hláška
     * @param event Daná udalosť
     */
    public abstract void chyba(TahEvent event);

    /**
     * Volaná pri prijatí události typu šach
     * @param event Daná udalosť
     */
    public abstract void sach(TahEvent event);

    /**
     * Volaná pri prijatí události typu mat
     * @param event Daná udalosť
     */
    public abstract void mat(TahEvent event);

    /**
     * Pridá daný objekt do kolekcie listenerov
     * @param l Daný objekt
     */
    synchronized public void addTahListener(TahListener l) {
        if (listeners == null)
            listeners = new Vector();
        listeners.addElement(l);
    }


    /**
     * Odoberie daný objekt zo kolekcie listenerov
     * @param l Daný objekt
     */
    synchronized public void removeTahListener(TahListener l) {
        if (listeners == null)
            listeners = new Vector();
        listeners.removeElement(l);
    }

    /**
     * Odošle udalosť s danými parametrami
     * @param odkal Bod odkiaľ hráč tiahol
     * @param kam Bod kam hráč tiahol
     * @param farba Farba figuriek hráča
     * @param type Typ figúrky hráča
     * @param typ Typ udalosti
     */
    private void fireEvent(Point odkal, Point kam, Figurka.Color farba, Figurka.Type type, TahEvent.EventType typ) {
        fireEvent(new TahEvent(this, odkal, kam, farba, type, typ));
    }

    /**
     * Odošle danú udalosť
     * @param event Daná udalosť
     */
    private void fireEvent(TahEvent event) {
        if (listeners != null && !listeners.isEmpty()) {
            this.event = event;

            Vector targets;
            synchronized (this) {
                targets = (Vector) listeners.clone();
            }

            Enumeration e = targets.elements();
            while (e.hasMoreElements()) {
                TahListener l = (TahListener) e.nextElement();
                if(event.getTyp() == TahEvent.EventType.tah){
                    l.tah(event);
                } else if(event.getTyp() == TahEvent.EventType.chyba){
                    l.chybyTah(event);
                } else if(event.getTyp() == TahEvent.EventType.vyber){
                    l.vyberFigurku(event);
                } else if(event.getTyp() == TahEvent.EventType.chybovaHlaska) {
                    l.chyba(event);
                } else if(event.getTyp() == TahEvent.EventType.sach) {
                    l.sach(event);
                } else if(event.getTyp() == TahEvent.EventType.mat) {
                    l.mat(event);
                }
            }
        }
    }

    /**
     * Pošle udalosť typu ťah
     * @param odkal Bod odkiaľ hráč tiahol
     * @param kam Bod kam hráč tiahol
     * @param farba Farba figuriek hráča
     */
    protected void fireTah(Point odkal, Point kam, Figurka.Color farba){
        fireEvent(new TahEvent(this, odkal, kam, farba, null, TahEvent.EventType.tah));
    }

    /**
     * Pošle udalosť typu chybný ťah
     * @param odkal Bod odkiaľ hráč tiahol
     * @param kam Bod kam hráč tiahol
     * @param farba Farba figuriek hráča
     */
    protected void fireChybnyTah(Point odkal, Point kam, Figurka.Color farba){
        fireEvent(new TahEvent(this, odkal, kam, farba, null, TahEvent.EventType.chyba));
    }

    /**
     * Odošle udalosť typu výber figúrky
     * @param odkal Bod odkiaľ hráč tiahol
     * @param kam Bod kam hráč tiahol
     * @param typ Typ zvolenej figúrky
     * @param farba Farba figuriek hráča
     */
    protected void fireVyberFigurku(Point odkal, Point kam, Figurka.Type typ, Figurka.Color farba){
        fireEvent(odkal, kam, farba, typ, TahEvent.EventType.vyber);
    }

    /**
     * Odošle udalosť typu chybová hláška
     * @param hlaska Chybová hláška
     */
    protected void fireChyba(String hlaska){
        fireEvent(new TahEvent(this, hlaska, TahEvent.EventType.chybovaHlaska));
    }

    /**
     * Odošle udalosť typu šach
     * @param farba Farba figúriek hráča, ktorý dostal šach
     */
    protected void fireSach(Figurka.Color farba){
        fireEvent(new TahEvent(this, farba, TahEvent.EventType.sach));
    }

    /**
     * Odošle udalosť typu mat
     * @param farba Farba figúriek hráča, ktorý dostal mat
     */
    protected void fireMat(Figurka.Color farba){
        fireEvent(new TahEvent(this, farba, TahEvent.EventType.mat));
    }
}
