package sach.file;

import sach.eventmodel.TahEvent;
import sach.logic.Figurka;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.Date;
import sach.ComponentInterface;
import sach.eventmodel.TahHandler;
import sach.logic.Point;

/**
 * Trieda používaná na zápis ťahov do súborov
 * @author Adam Netočný
 */
public class ZapisTahov extends TahHandler implements ComponentInterface {
    /**
     * Odkaz na FileWriter, ktorý spravuje súbor počas celej hry
     */
    private FileWriter fw;
    
    /**
     * Zaznam o súbore, do ktorého sa zapisuje
     */
    private File f;

    /**
     * Vytvorí nový súbor na zápis ťahov a priradí ho FileWriteru
     */
    public ZapisTahov() {
        try {
            SimpleDateFormat dates = new SimpleDateFormat("dd_M_yyyy_HH_mm_ss");
            Date date = new Date();
            File parDir = new File(new File("").getAbsolutePath() + File.separator + "Zaznam hier");
            if(!parDir.exists()){
                try {
                parDir.mkdir();
                } catch(Exception ex){}
            }
            f = new File(parDir.getAbsolutePath() + File.separator + "Hra_zacata_" + dates.format(date) + ".txt");
            if (!f.exists()) {
                try {
                    f.createNewFile();
                } catch(Exception ex){}
            }
            fw = new FileWriter(f);
        } catch (Exception ex) {

        }
    }

    /**
     * Zapíše ťah do súboru
     * @param z Číselná pozícia políčka odkiaľ hráč tiahol
     * @param kam Číselná pozícia políčka kam hráč tiahol
     * @param farba Farba hráča, ktorý tiahol
     */
    private void zapisTah(Point z, Point kam, Figurka.Color farba){
        try {
            fw.write(z + " -> " + kam +
                    "; Farba: " + ((farba==Figurka.Color.biela)? "biela" : "cierna") + "\r\n");
        } catch (IOException ex) {}
    }

    /**
     * Zapíše výber figúrky do súboru
     * @param typ Typ zapisovanej figúrky
     */
    private void zapisVyber(Figurka.Type typ){
        try {
            fw.write("Uzivatel zvolil figurku typu: " + typ.toString() + "\r\n");
        } catch (IOException ex) {}
    }

    /**
     * Zapíše prostý text do súboru
     * @param text Text, ktorý sa má zapísať
     */
    private void zapisText(String text){
        try {
            fw.write(text + "\r\n");
        } catch (IOException ex) {}
    }

    /**
     * Uzavrie súbor a zruší objekt
     * @throws java.lang.Throwable
     */
    @Override
    protected void finalize() throws Throwable {
        fw.close();
        super.finalize();
    }

    /**
     * Získava absolútnu cestu k súboru, do ktorého sa zapisuje
     * @return Absolútna cesta k súboru, do ktorého sa zapisuje
     */
    public String getFilePath(){
        return f.getAbsolutePath();
    }

    /**
     * Volaná na komponente pri ukončovaní programu
     */
    public void exit(){
        try {
            finalize();
        } catch (Throwable ex) {
        }
    }

    /**
     * Volaná pri prijatí události typu ťah
     * @param event Daná udalosť
     */
    @Override
    public void tah(TahEvent event) {
        if(!event.equals(this.event)){
            zapisTah(event.getOdkial(), event.getKam(), event.getFarba());
            fireTah(event.getOdkial(), event.getKam(), event.getFarba());
        }
    }

    /**
     * Volaná pri prijatí události typu chybný ťah
     * @param event Daná udalosť
     */
    @Override
    public void chybyTah(TahEvent event) {
        if(!event.equals(this.event)){
            fireChybnyTah(event.getOdkial(), event.getKam(), event.getFarba());
        }
    }

    /**
     * Volaná pri prijatí události typu výber figúrky
     * @param event Daná udalosť
     */
    @Override
    public void vyberFigurku(TahEvent event) {
        if(!event.equals(this.event)){
            if(event.getTypFig() != null){
                zapisVyber(event.getTypFig());
            }
            fireVyberFigurku(event.getOdkial(), event.getKam(), event.getTypFig(), event.getFarba());
        }
    }

    /**
     * Volaná pri prijatí události typu chybová hláška
     * @param event Daná udalosť
     */
    @Override
    public void chyba(TahEvent event) {
        if(!event.equals(this.event)){
            zapisText("\tChyba: " + event.getChyba());
            fireChyba(event.getChyba());
        }
    }

    /**
     * Volaná pri prijatí události typu šach
     * @param event Daná udalosť
     */
    @Override
    public void sach(TahEvent event) {
        if(!event.equals(this.event)){
            zapisText("\t" + ((event.getFarba() == Figurka.Color.biela)? "Biely" : "Cierny")
                + " kral dostal sach!");
            fireSach(event.getFarba());
        }
    }

    /**
     * Volaná pri prijatí události typu mat
     * @param event Daná udalosť
     */
    @Override
    public void mat(TahEvent event) {
        if(!event.equals(this.event)){
            zapisText("\t" + ((event.getFarba() == Figurka.Color.biela)? "Biely" : "Cierny")
                + " kral dostal mat!");
            fireMat(event.getFarba());
        }
    }
}
