package sach.gui;

import sach.eventmodel.TahEvent;
import sach.logic.Figurka;
import sach.*;
import java.awt.*;
import sach.logic.Point;
import java.awt.event.*;
import javax.swing.*;
import sach.eventmodel.TahHandler;
import sach.logic.FigurkaInterface;

/**
 * Panel zobrazujúci aktuálne rozloženie figúriek na hracej ploche
 * @author Adam Netočný
 */
class FigurkyPanel extends TahHandler {
    /**
     * Odkaz na okno, v ktorom je panel vykreslený
     */
    private MainFrame okno;

    /**
     * Čakacie okno
     */
    private SendReciveDialog dialog;

    /**
     * Vykreslovaný panel
     */
    private JPanel panel;

    /**
     * Logicka hodnota, ci ma byt hrac na tahu
     */
    private boolean naTahu = false;

    /**
     * Načúva kliknutiam na prvkoch JLabel
     */
    private Listener lst = new Listener(this);

    /**
     * Ukladá aktuálne rozostavenie figúriek na hracej ploche
     */
    private FigurkaInterface[][] rozostavenie;

    /**
     * Detekuje, že v hre nastal mat
     */
    private boolean mat;

    /**
     * Vytvorí a pridá panel podľa rozloženia figúriek
     * @param okno Odkaz na okno, do ktorého má byť panel pridaný
     */
    public FigurkyPanel(final MainFrame okno) {
        panel = new JPanel();
        dialog = new SendReciveDialog();

        this.okno = okno;
        final JPanel content = (JPanel)okno.getContentPane();
        content.setLayout(null);

        panel.setLayout(new GridLayout(0, 8, 0, 0));
        panel.setMaximumSize(new Dimension(440, 440));
        panel.setOpaque(false);

        drawFigurky();

        content.add(panel);

        panel.setBounds(33, 31, 440, 440);
    }

    /**
     * Prekreslí figúrky na paneli podľa aktualného rozloženia figúriek
     */
    protected void drawFigurky(){
        rozostavenie = SachApp.gameStatus.getSachovnica();
        panel.removeAll();

        JLabel figurka = new JLabel(" ");
        
        for (int a = 0; a<64; a++){
            JLabel b = new JLabel(" ");
            b.addMouseListener(lst);
            if(SachApp.gameStatus.getPlayerColor() == Figurka.Color.biela){
                b.setToolTipText(new Point(a).toString());
            } else {
                b.setToolTipText(new Point(63-a).toString());
            }
            panel.add(b);
        }

         for(int a = 0; a < rozostavenie.length; a++){
            for(int b = 0; b < rozostavenie[a].length; b++){
                if(rozostavenie[a][b]!=null){
                    if(rozostavenie[a][b].getType()==Figurka.Type.veza){
                        figurka = new JLabel(createImageIcon(((rozostavenie[a][b].getColor()
                                == Figurka.Color.biela)? "biele/" : "cierne/") + "veza.png"));
                    } else if(rozostavenie[a][b].getType()==Figurka.Type.kon){
                        figurka = new JLabel(createImageIcon(((rozostavenie[a][b].getColor()
                                == Figurka.Color.biela)? "biele/" : "cierne/") + "kon.png"));
                    } else if(rozostavenie[a][b].getType()==Figurka.Type.strelec){
                        figurka = new JLabel(createImageIcon(((rozostavenie[a][b].getColor()
                                == Figurka.Color.biela)? "biele/" : "cierne/") + "strelec.png"));
                    } else if(rozostavenie[a][b].getType()==Figurka.Type.kral){
                        figurka = new JLabel(createImageIcon(((rozostavenie[a][b].getColor()
                                == Figurka.Color.biela)? "biele/" : "cierne/") + "kral.png"));
                    } else if(rozostavenie[a][b].getType()==Figurka.Type.dama){
                        figurka = new JLabel(createImageIcon(((rozostavenie[a][b].getColor()
                                == Figurka.Color.biela)? "biele/" : "cierne/") + "dama.png"));
                    } else if(rozostavenie[a][b].getType()==Figurka.Type.pesiak){
                        figurka = new JLabel(createImageIcon(((rozostavenie[a][b].getColor()
                                == Figurka.Color.biela)? "biele/" : "cierne/") + "pesiak.png"));
                    }

                    figurka.addMouseListener(lst);
                    figurka.setToolTipText(new Point(a, b).toString());
                    if(SachApp.gameStatus.getPlayerColor() == Figurka.Color.biela){
                        panel.remove((a*8)+b);
                        panel.add(figurka, (a*8)+b);
                    } else {
                        panel.remove(63-((a*8)+b));
                        panel.add(figurka, 63-((a*8)+b));
                    }
                }
            }
        }
        panel.validate();
    }

    /**
     * Prekreslí figúrky
     */
    public void validate() {
        panel.validate();
        drawFigurky();
    }

    /**
     * Získa adresu zdroja a výtvorí z neho ImageIcon
     * @param path Lokálna adresa zdroja
     * @return ImageIcon vytvorený z daného zdroja
     */
    private ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = SachApp.class.getResource("resources/" + path);
        return new ImageIcon(imgURL);
    }

    /**
     * Volaná pri prijatí události typu ťah
     * @param event Daná udalosť
     */
    @Override
    public void tah(TahEvent event) {
        if(!event.equals(this.event)){
            cakaj(false);
            
            fireTah(event.getOdkial(), event.getKam(), event.getFarba());
        } else {
            cakaj(true);
        }
        drawFigurky();
    }

    /**
     * Volaná pri prijatí události typu chybný ťah
     * @param event Daná udalosť
     */
    @Override
    public void chybyTah(TahEvent event) {
        if(!event.equals(this.event)){
            fireChybnyTah(event.getOdkial(), event.getKam(), event.getFarba());
        }
        cakaj(false);
        drawFigurky();
    }

    /**
     * Volaná pri prijatí události typu výber figúrky
     * @param event Daná udalosť
     */
    @Override
    public void vyberFigurku(TahEvent event) {
        if(!event.equals(this.event)){
            if(event.getTypFig() == null){
                if(!dialog.isVisible()){
                    ChooseFigurka d = new ChooseFigurka(event.getFarba());
                    d.setVisible(true);
                    naTahu = false;
                    fireVyberFigurku(event.getOdkial(), event.getKam(), d.getResult(), event.getFarba());
                    d.dispose();
                } else {
                    naTahu = true;
                    fireVyberFigurku(event.getOdkial(), event.getKam(), event.getTypFig(), event.getFarba());
                }
            } else {
                fireVyberFigurku(event.getOdkial(), event.getKam(), event.getTypFig(), event.getFarba());
            }
        } else {
            if(naTahu){
                cakaj(false);
            } else {
                cakaj(true);
            }
        }
        drawFigurky();
        try {
            Thread.sleep(200);
        } catch (InterruptedException ex) {}
        drawFigurky();
    }

    /**
     * Volaná pri prijatí události typu chybová hláška
     * @param event Daná udalosť
     */
    @Override
    public void chyba(TahEvent event) {
        if(!mat){
            if(!event.equals(this.event)){
                fireChyba(event.getChyba());
            }
            JOptionPane.showMessageDialog(MainFrame.me, event.getChyba(), "Sach", JOptionPane.ERROR_MESSAGE);
            SachApp.me.exit();
        }
    }

    /**
     * Volaná pri prijatí události typu šach
     * @param event Daná udalosť
     */
    @Override
    public void sach(TahEvent event) {
        if(!mat){
            if(!event.equals(this.event)){
                fireSach(event.getFarba());
            }
            JOptionPane.showMessageDialog(MainFrame.me, ((event.getFarba() == Figurka.Color.biela)? "Biely" : "Cierny")
                    + " kral dostal sach!", "Sach", JOptionPane.WARNING_MESSAGE);
        }
    }

    /**
     * Volaná pri prijatí události typu mat
     * @param event Daná udalosť
     */
    @Override
    public void mat(TahEvent event) {
        mat = true;
        SachApp.gameStatus.setGameSterted(false);
        cakaj(false);
        if(!event.equals(this.event)){
            fireMat(event.getFarba());
        }

        JOptionPane.showMessageDialog(MainFrame.me, ((event.getFarba() == Figurka.Color.biela)? "Biely" : "Cierny")
                + " kral dostal mat!", "Sach", JOptionPane.WARNING_MESSAGE);

        try {
            Thread.sleep(1000);
        } catch (InterruptedException ex) {}

        SachApp.me.exit();
    }

    /**
     * Uvedie/Vyvedie okno do/zo stavu čakania(tj. čakacie okno plus uzamknutie funkcií hlavného okna
     * @param otvor true - uzamknúť; false - odomknúť
     */
    protected void cakaj(boolean otvor){
        SachApp.gameStatus.setGameSterted(!otvor);
        okno.suspend(otvor);
        dialog.setVisible(otvor);
        drawFigurky();
    }

    //<editor-fold defaultstate="collapsed" desc="Listenery">
    /**
     * Trieda Listener načúvajúca eventom na prvkoch JLabel na vykreslenom hracom poli
     */
    private class Listener extends MouseAdapter {
        /**
         * Detekuje prvý klik myšou
         */
        private boolean clicked = false;
        /**
         * Pozícia, na ktorú užívateľ klikol
         */
        private Point pos;
        /**
         * Odkaz na objekt šachovnice
         */
        private FigurkyPanel panel;
        /**
         * Odkaz na predchádzajúce poličko, na ktoré užívateľ klikol
         */
        private JLabel old;

        /**
         * Vytvorí MouseAdapter pričom je naviac uložený odkaz na objekt Figurky panel
         * @param sach Odkaz na objekt Figurky panel
         */
        public Listener(FigurkyPanel sach) {
            super();
            this.panel = sach;
        }

        /**
         * Načuva dvojici kliknutí, pričom pri druhom kliknutí spustí metódu tah pre vykonanie ťahu
         * @param e Zdedené:)
         */
        @Override
        public void mouseClicked(MouseEvent e) {
            if(e.getButton() == MouseEvent.BUTTON1 && SachApp.gameStatus.getGameStarted()){
                JLabel lbl = (JLabel)e.getSource();
                if(clicked){
                    lbl.setOpaque(true);
                    lbl.setBackground(Color.blue);

                    Point pos1 = new Point(lbl.getToolTipText());
                    clicked = false;
                    
                    fireTah(pos, pos1, SachApp.gameStatus.getPlayerColor());
                    
                    old.setOpaque(false);
                    old.setBackground(null);
                    lbl.setOpaque(false);
                    lbl.setBackground(null);

                    panel.drawFigurky();
                } else {
                    pos = new Point(lbl.getToolTipText());

                    int x = pos.getX();
                    int y = pos.getY();
                    if(rozostavenie[x][y] != null){
                        if(rozostavenie[x][y].getColor() == SachApp.gameStatus.getPlayerColor()){
                            clicked = true;

                            lbl.setOpaque(true);
                            lbl.setBackground(Color.blue);

                            old = lbl;
                        }
                    }
                }
            }
        }
    }
    //</editor-fold>
}
