package sach.gui;

import sach.*;
import java.awt.event.*;
import javax.swing.*;
import sach.eventmodel.*;

/**
 * Hlavné okno aplikácie
 * @author Adam Netočný
 */
public class MainFrame extends JFrame implements ComponentInterface {
    /**
     * Panel zobrazujúci aktuálne rozloženie figúriek na hracej ploche
     */
    final protected FigurkyPanel panel;

    /**
     * Panel zobrazujúci aktuálne rozloženie figúriek na hracej ploche
     */
    final protected MainMenu menu;
    
    /**
     * Odkaz na dialóg About...
     */
    protected AboutDialog aboutdialog;

    /**
     * Odkaz na dialóg pre vytvorenie servru
     */
    protected ServerClientDialog serverdialog;

    /**
     * Odkaz na dialóg pre vytvorenie klienta
     */
    protected ServerClientDialog clientdialog;

    /**
     * Statický odkaz na hlavné okno programu
     */
    protected static MainFrame me;

    /**
     * Vytvorí hlavné okno aplikácie
     */
    public MainFrame() {
        super("Šach");
        me = this;
        panel = new FigurkyPanel(this);

        getContentPane().setLayout(null);
        //sachovnica na pozadie
        JLabel sachovnica = new JLabel();
        sachovnica.setIcon(createImageIcon("chessboard.jpg"));      

        //pridanie sachovnice
        this.getContentPane().add(sachovnica);
        sachovnica.setBounds(0, 0, 504, 500);

        this.menu = (MainMenu) createMenu();
        //pridanie menu
        this.setJMenuBar(menu);

        this.setBounds(20, 20, 510, 548);
        this.setLocationRelativeTo(null);
        this.setResizable(false);
        this.setIconImage(createImageIcon("biele/kon.png").getImage());

        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                SachApp.me.exit();
            }
        });
        setVisible(true);
    }

    /**
     * Pridá daný objekt do kolekcie listenerov
     * @param l Daný objekt
     */
    public void addTahListener(TahListener l) {
        panel.addTahListener(l);
    }

    @Override
    public void dispose() {
        super.dispose();      
    }

    /**
     * Vráti odkaz na objekt FigurkyPanel(TahListener pre GUI)
     * @return odkaz na objekt FigurkyPanel(TahListener pre GUI)
     */
    public TahListener getTahListener() {
        return panel;
    }

    /**
     * Vytvorí menu pre aplikáciu
     * @return Novovytvorené menu
     */
    private JMenuBar createMenu(){
        MainMenu mainmen = new MainMenu();
        return mainmen;
    }

    /**
     * Získa adresu zdroja a výtvorí z neho ImageIcon
     * @param path Lokálna adresa zdroja
     * @return ImageIcon vytvorený z daného zdroja
     */
    private ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = SachApp.class.getResource("resources/" + path);
        return new ImageIcon(imgURL);
    }

    /**
     * Uzamkne funkcie hlavného okna okrem zobrazenia vyhodených figúriek a helpu
     * @param ts true - uzamknúť; false - odomknúť
     */
    protected void suspend(boolean ts) {
        WindowListener[] list = getWindowListeners().clone();
        for(int a = 0; a < list.length; a++){
            removeWindowListener(list[a]);
        }
        if(ts) {
            addWindowListener(new WindowAdapter() { });
            menu.exit.setEnabled(false);
        } else {
            addWindowListener(new WindowAdapter() {
                @Override
                public void windowClosing(WindowEvent e) {
                    SachApp.me.exit();
                }
            });
            menu.exit.setEnabled(true);
        }
    }

    /**
     * Ruší zdroje
     */
    public void exit(){
        if(aboutdialog!=null){
            aboutdialog.dispose();
        }
        if(serverdialog!=null){
            serverdialog.dispose();
        }
        if(clientdialog!=null){
            clientdialog.dispose();
        }
        setVisible(false);
        dispose();
    }
}
