package sach.gui;

import sach.logic.Figurka;
import sach.network.Client;
import sach.network.Server;
import sach.network.NetworkGameInterface;
import sach.*;
import java.awt.event.*;
import java.net.*;
import javax.swing.*;

/**
 * Hlavné menu aplikácie
 * @author Adam Netočný
 */
class MainMenu extends JMenuBar {
    /**
     * Detekuje, či je otvorené dialógové okno s vyhodenými figúrkami
     */
    public static boolean vyhodOtvoreny = false;

    /**
     * Odkaz na položku File v menu
     */
    private JMenu menu;

    /**
     * Odkaz na položku novej hry v menu
     */
    private JMenu newGame;

    /**
     * Odkaz na položku vytvorenia servru v menu
     */
    private final JMenuItem server;

    /**
     * Odkaz na položku vytvorenia klienta v menu
     */
    private final JMenuItem client;

    /**
     * Odkaz na položku ukončenia programu v menu
     */
    protected JMenuItem exit;

    /**
     * Odkaz na položku zobrazenia v menu
     */
    private JMenu view;

    /**
     * Odkaz na položku zobrazenia vyhodených figúriek v menu
     */
    private JMenuItem vyhodene;

    /**
     * Odkaz na položku ? v menu
     */
    private JMenu about;

    /**
     * Odkaz na položku o programe v menu
     */
    private JMenuItem aboutItem;

    /**
     * Odkaz na položku javadoc v menu
     */
    private JMenuItem help;
    
    /**
     * Vytvorí hlavné menu aplikácie
     */
    public MainMenu() {
        menu = new JMenu("File");

        exit = new JMenuItem("Exit");
        exit.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                SachApp.me.exit();
            }
        });

        newGame = new JMenu("New Game");
        server = new JMenuItem("Create server");
        client = new JMenuItem("Join game");
        server.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                SachApp.gameStatus.setPlayerColor( Figurka.Color.biela);
                Server srv = new Server();
                SachApp.me.setNetwork(srv);
                MainFrame.me.serverdialog = new ServerClientDialog(ServerClientDialog.Type.server);
                if(createClientServer(srv, MainFrame.me.serverdialog, false)){
                    JOptionPane.showMessageDialog(MainFrame.me, "Boli ste uspesne prihlaseny!\n" +
                            "Hra zacina. Kliknite na tlacitko ok.",
                        "Nova hra", JOptionPane.INFORMATION_MESSAGE);

                    MainFrame.me.panel.drawFigurky();
                    MainFrame.me.panel.cakaj(false);

                    server.setEnabled(false);
                    client.setEnabled(false);
                } else {
                    SachApp.gameStatus.setPlayerColor( Figurka.Color.biela);
                    JOptionPane.showMessageDialog(MainFrame.me, "Spojenie sa nepodarilo vytvorit.",
                        "Nova hra", JOptionPane.WARNING_MESSAGE);
                }
                MainFrame.me.serverdialog.dispose();
            }
        });
        client.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                SachApp.gameStatus.setPlayerColor(Figurka.Color.cierna);
                Client clt = new Client();
                SachApp.me.setNetwork(clt);
                MainFrame.me.clientdialog = new ServerClientDialog(ServerClientDialog.Type.client);
                if(createClientServer(clt, MainFrame.me.clientdialog, true)){
                    JOptionPane.showMessageDialog(MainFrame.me, "Boli ste uspesne prihlaseny!\n" +
                            "Hra zacina. Kliknite na tlacitko ok.",
                        "Nova hra", JOptionPane.INFORMATION_MESSAGE);
                    
                    MainFrame.me.panel.drawFigurky();
                    MainFrame.me.panel.cakaj(true);
                    
                    server.setEnabled(false);
                    client.setEnabled(false);
                    
                    //SachApp.okno.sach.waitForFirstMove();
                    MainFrame.me.panel.drawFigurky();
                } else {
                    SachApp.gameStatus.setPlayerColor( Figurka.Color.biela);
                    JOptionPane.showMessageDialog(MainFrame.me, "Spojenie sa nepodarilo vytvorit.",
                        "Nova hra", JOptionPane.WARNING_MESSAGE);
                }
                MainFrame.me.clientdialog.dispose();
            }
        });

        about = new JMenu("?");
        aboutItem = new JMenuItem("About Šach");
        aboutItem.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                MainFrame.me.aboutdialog = new AboutDialog();
                MainFrame.me.aboutdialog.setVisible(true);
                MainFrame.me.aboutdialog.dispose();
            }
        });
        help = new JMenuItem("Help a JavaDoc");
        help.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JavaDocBrowser okno = new JavaDocBrowser();
                okno.setVisible(true);
                okno.dispose();
            }
        });

        view = new JMenu("View");
        vyhodene = new JMenuItem("Vyhodene figurky");
        vyhodene.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if(!vyhodOtvoreny){
                    VyhodeneDialog okno = new VyhodeneDialog();
                    okno.setVisible(true);
                    vyhodOtvoreny = true;
                }
            }
        });

        view.add(vyhodene);

        //pridame polozky novej hry
        newGame.add(server);
        newGame.add(client);

        //pridame polozku nava hra a exit
        menu.add(newGame);
        menu.addSeparator();
        menu.add(exit);

        //pridame polozky pre help
        about.add(help);
        about.addSeparator();
        about.add(aboutItem);

        //pridame polozky menu
        add(menu);
        add(view);
        add(Box.createHorizontalGlue());
        add(about);
    }

    /**
     * Vytvára pomocou vlákna pripojenie
     * @param srvOz Odkaz na sieťové rozhrannie hry
     * @param srvDlg Odkaz na dialóg zobrazovaný počas vytvárania
     * @param clienserver true - vytvorí clienta; false - vytvorí server
     * @return true - úspešne pripojené; false - nepripojené
     */
    private static boolean createClientServer(NetworkGameInterface srvOz, ServerClientDialog srvDlg, boolean clienserver){
        final NetworkGameInterface srv = srvOz;
        final ServerClientDialog serverdialog = srvDlg;
        final boolean client = clienserver;
        String ip1 = "";
        if(client){
                IPChooser ipch = new IPChooser();
                ipch.setVisible(true);
                ip1 = ipch.getIPHost();
                if(ipch != null){
                    ipch.dispose();
                }
                if(ip1 != null) {
                    if(ip1.equals("")){
                        try {
                            InetAddress here = InetAddress.getLocalHost();
                            ip1 = here.getHostAddress();
                        } catch (UnknownHostException ex) {}
                    }
                } else {
                    try {
                        InetAddress here = InetAddress.getLocalHost();
                        ip1 = here.getHostAddress();
                    } catch (UnknownHostException ex) {}
                }
        } else {
            ip1 = "";
        }

        final String ip = ip1;
        try {
            class Vlakno extends Thread {
                @Override
                public void run() {
                    try {
                        srv.connect(ip, 8190);
                        if(client){
                            serverdialog.setHostText(srv.getHostName());
                            try {
                                Thread.sleep(1000);
                            } catch (InterruptedException ex) {
                                JOptionPane.showMessageDialog(MainFrame.me, ex.getMessage(),
                                        "Chyba programu", JOptionPane.ERROR_MESSAGE);
                                srv.exit();
                            }
                        }
                    } catch (Throwable ex) {
                        JOptionPane.showMessageDialog(MainFrame.me, ex.getMessage(),
                                "Chyba programu", JOptionPane.ERROR_MESSAGE);
                        try {
                            srv.exit();
                        } catch (Throwable ex1) {
                            ex1.printStackTrace();
                        }
                    }
                    serverdialog.setVisible(false);
                }
            }

            if(client){
                try {
                    InetAddress here = InetAddress.getByName(ip);

                    String ipAddress = here.getHostAddress();

                    serverdialog.setIPText(ipAddress);
                } catch (final Exception ex){
                    throw new Exception(){
                        @Override
                        public String getMessage() {
                            return "Nespravna adresa: " + ex.getMessage();
                        }
                    };
                }
            } else {
                InetAddress here = InetAddress.getLocalHost();
                String hostName = here.getHostName();
                String ipAddress = here.getHostAddress();
                serverdialog.setIPHostText(ipAddress, hostName);
            }
            Thread vlakno = new Thread(new Vlakno());
            vlakno.start();
         
            serverdialog.setVisible(true);
            Thread.sleep(100);
            
            if(vlakno.isAlive()){
                vlakno.interrupt();
                srv.exit();
                return false;
            }

            if(srv.isConnected()){
                return true;
            } else {
                srv.exit();
                return false;
            }
        } catch (Throwable ex) {
            JOptionPane.showMessageDialog(MainFrame.me, ex.getMessage(),
                    "Chyba programu", JOptionPane.ERROR_MESSAGE);
            try {
                srv.exit();
            } catch (Exception ex1) {
                ex.printStackTrace();
            }
            return false;
        }
    }
}
