package sach.gui;

import sach.*;
import java.awt.event.*;
import javax.swing.*;
/**
 * Dialógové okno zobrazované pri vytváraní servru/klienta
 * @author Adam Netočný
 */
class ServerClientDialog extends JDialog {
    /**
     * Odkaz na prvok JLabel zobrazujúci IP adresu servru
     */
    private JLabel ip;

    /**
     * Odkaz na prvok JLabel zobrazujúci host name servru
     */
    private JLabel host;

    /**
     * Reprezentuje typ vytvaraného pripojenia
     */
    public static enum Type {
        /**
         * Vytvára sa klientské pripojenie
         */
        client,
        /**
         * Vytvára sa server
         */
        server
    }

    /**
     * Vytvorí dialóg podľa typu vytvaraného pripojenia
     * @param type Typ vytvaraného pripojenia
     */
    public ServerClientDialog(Type type) {
        super(MainFrame.me, (type == Type.server)? "Waiting for connections...." : "Connecting....", true);
        
        this.setBounds(135, 100, 250, 165);
        setLocationRelativeTo(MainFrame.me);
        setResizable(false);
        setIconImage(createImageIcon("biele/kon.png").getImage());
        setLayout(null);

        final JDialog me = this;

        JLabel txt = new JLabel((type == Type.server)? "Čaká sa na " +
                "pripojenie...." : "Pripaja sa k servru....");
        ip = new JLabel("IP adresa servru: ");
        host = new JLabel("Hostname servru: ");

        JProgressBar pgb = new JProgressBar();
        pgb.setValue(0);

        JButton btn = new JButton("Cancel");
        btn.getInputMap().put(KeyStroke.getKeyStroke("SPACE"),
                            "none");
        btn.getInputMap().put(KeyStroke.getKeyStroke("released SPACE"),
                            "none");
        btn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                me.setVisible(false);
            }
        });

        add(txt);
        add(ip);
        add(host);
        add(pgb);
        add(btn);

        txt.setBounds(5, 5, 240, 20);
        txt.setHorizontalAlignment(JLabel.CENTER);

        ip.setBounds(5, 30, 240, 20);
        ip.setHorizontalAlignment(JLabel.CENTER);

        host.setBounds(5, 55, 240, 20);
        host.setHorizontalAlignment(JLabel.CENTER);

        pgb.setBounds(5, 80, 230, 20);
        pgb.setIndeterminate(true);

        btn.setBounds(155, 105, 80, 25);

    }

    /**
     * Nastaví text v poli pre IP adresu
     * @param ip Text, ktorý má byť vložený do poľa; predpokladá sa IP adresa
     */
    public void setIPText(String ip){
        this.ip.setText("IP adresa servru: " + ip);
    }

    /**
     * Nastaví text v poli pre Host Name
     * @param host Text, ktorý má byť vložený do poľa; predpokladá sa Host Name
     */
    public void setHostText(String host){
        this.host.setText("Hostname servru: " + host);
    }

    /**
     * Nastaví text v poliach pre IP adresu a pre Host Name
     * @param ip Text, ktorý má byť vložený do poľa; predpokladá sa IP adresa
     * @param host Text, ktorý má byť vložený do poľa; predpokladá sa Host Name
     */
    public void setIPHostText(String ip, String host){
        setIPText(ip);
        setHostText(host);
    }

    /**
     * Získa adresu zdroja a výtvorí z neho ImageIcon
     * @param path Lokálna adresa zdroja
     * @return ImageIcon vytvorený z daného zdroja
     */
    private ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = SachApp.class.getResource("resources/" + path);
        return new ImageIcon(imgURL);
    }
}
