package sach.gui;

import sach.logic.Figurka;
import sach.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Dialógové okno zobrazujúce vyhodené figúrky
 * @author Adam Netočný
 */
class VyhodeneDialog extends JDialog {
    /**
     * Odkaz na panel obsahujúci biele figúrky
     */
    private JPanel panelbiele;

    /**
     * Odkaz na panel obsahujúci čierne figúrky
     */
    private JPanel panelcierne;

    /**
     * Odkaz na timer, ktorý obnovuje náhľad
     */
    private Timer timer;

    /**
     * Odkaz na posúvnik pre biele figúrky
     */
    private JScrollPane posuvnikbiele;

    /**
     * Odkaz na posúvnik pre čierne figúrky
     */
    private JScrollPane posuvnikcierne;

    /**
     * Vytvorí dialóg
     */
    public VyhodeneDialog() {
        super(MainFrame.me, "Vyhodene figurky", false);
        
        this.setBounds(135, 100, 270, 305);
        setLocationRelativeTo(MainFrame.me);
        setResizable(false);
        setAlwaysOnTop(false);
        setIconImage(createImageIcon("biele/kon.png").getImage());


        panelbiele = new JPanel(new GridLayout(0, 2));
        panelcierne = new JPanel(new GridLayout(0, 2));
        panelbiele.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        panelcierne.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        
        setLayout(new GridLayout(0, 2));
        posuvnikbiele = new JScrollPane(panelbiele, 
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        posuvnikcierne = new JScrollPane(panelcierne,
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        
        add(posuvnikbiele, 0);
        add(posuvnikcierne, 1);

        int delay = 1000; //milliseconds
        ActionListener taskPerformer = new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                panelbiele.removeAll();
                for(int a=0; a<SachApp.gameStatus.getKontajner().getBieleLength(); a++){
                    JLabel figurka;
                    if(SachApp.gameStatus.getKontajner().getBieluAt(a)!=null){
                        Figurka.Type typ = SachApp.gameStatus.getKontajner().getBieluAt(a).getType();
                        if(typ == Figurka.Type.kon){
                             figurka = new JLabel(createImageIcon("biele/kon.png"));
                        } else if(typ == Figurka.Type.veza){
                            figurka = new JLabel(createImageIcon("biele/veza.png"));
                        } else if(typ == Figurka.Type.strelec){
                            figurka = new JLabel(createImageIcon("biele/strelec.png"));
                        } else if(typ == Figurka.Type.dama){
                            figurka = new JLabel(createImageIcon("biele/dama.png"));
                        } else {
                            figurka = new JLabel(createImageIcon("biele/pesiak.png"));
                        }
                        panelbiele.add(figurka);
                    }
                }
                if(SachApp.gameStatus.getKontajner().getBieleLength() % 2 != 0){
                    panelbiele.add(new JLabel());
                }

                panelcierne.removeAll();
                for(int a=0; a<SachApp.gameStatus.getKontajner().getCierneLength(); a++){
                    JLabel figurka;
                    if(SachApp.gameStatus.getKontajner().getCiernuAt(a)!=null){
                        Figurka.Type typ = SachApp.gameStatus.getKontajner().getCiernuAt(a).getType();
                        if(typ == Figurka.Type.kon){
                             figurka = new JLabel(createImageIcon("cierne/kon.png"));
                        } else if(typ == Figurka.Type.veza){
                            figurka = new JLabel(createImageIcon("cierne/veza.png"));
                        } else if(typ == Figurka.Type.strelec){
                            figurka = new JLabel(createImageIcon("cierne/strelec.png"));
                        } else if(typ == Figurka.Type.dama){
                            figurka = new JLabel(createImageIcon("cierne/dama.png"));
                        } else {
                            figurka = new JLabel(createImageIcon("cierne/pesiak.png"));
                        }
                        panelcierne.add(figurka);

                    }
                }
                if(SachApp.gameStatus.getKontajner().getCierneLength() % 2 != 0){
                    panelcierne.add(new JLabel());
                }

                panelbiele.validate();
                panelcierne.validate();
                posuvnikbiele.validate();
                posuvnikcierne.validate();
                posuvnikbiele.getViewport().validate();
                posuvnikcierne.getViewport().validate();
                posuvnikbiele.getParent().validate();
            }
        };
        timer = new Timer(delay, taskPerformer);
        timer.start();

        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                MainMenu.vyhodOtvoreny = false;
                ((JDialog)e.getWindow()).dispose();
            }
        });
    }

    @Override
    public void dispose() {
        super.dispose();
        if(timer!=null){
            timer.stop();
        }
    }

    /**
     * Získa adresu zdroja a výtvorí z neho ImageIcon
     * @param path Lokálna adresa zdroja
     * @return ImageIcon vytvorený z daného zdroja
     */
    private ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = SachApp.class.getResource("resources/" + path);
        return new ImageIcon(imgURL);
    }
}
