package sach.logic;

/**
 * Implementácia figúrky typu dáma
 * @author Adam Netočný
 */
class Dama extends Figurka implements FigurkaInterface {
    /**
     * Vytvorí novú figúrku typu dáma
     * @param x X-ová pozícia, na ktorej je dáma umiestnená
     * @param y Y-ová pozícia, na ktorej je dáma umiestnená
     * @param farba Farba novovytvorenej figúrky
     */
    Dama(int x, int y, Figurka.Color farba){
        super(x, y, farba);
        typ = Figurka.Type.dama;
    }

    /**
     * Kontrola ťahu figúrkou typu dáma
     * @param x X-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param y Y-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param sach Udáva pole, v ktorom sú rozmiestnené figúrky pričom sa berie v úvahu aj rozmiestnenie ostatných figúriek
     * @param detKral Určuje či má, alebo nemá brať v úvahu, že figúrkou, na ktorú sa snaží preisť je kráľ
     * @return true, ak sa môže figúrka presunúť, inak vráti false
     */
    @Override
    public boolean tah(int x, int y, FigurkaInterface[][] sach, boolean detKral) {
        if((this.x<8 && this.x>=0) && (this.y<8 && this.y>=0) && (x<8 && x>=0) && (y<8 && y>=0)){
            if(Math.abs(this.x-x)==Math.abs(this.y-y)){
                if(getFigrkaDirectionXY(sach, this.y<y, this.x<x)==null){
                    return true;
                } else if(getFigrkaDirectionXY(sach, this.y<y, this.x<x).getX()==x
                        && getFigrkaDirectionXY(sach, this.y<y, this.x<x).getY()==y
                        && getFigrkaDirectionXY(sach, this.y<y, this.x<x).getColor()!=farba
                        && (getFigrkaDirectionXY(sach, this.y<y, this.x<x).getType()!=Figurka.Type.kral || !detKral)){
                    return true;
                } else if(((this.x<x) ?
                        getFigrkaDirectionXY(sach, this.y<y, this.x<x).getX()>x :
                        getFigrkaDirectionXY(sach, this.y<y, this.x<x).getX()<x)
                        && ((this.y<y)?
                        getFigrkaDirectionXY(sach, this.y<y, this.x<x).getY()>y :
                        getFigrkaDirectionXY(sach, this.y<y, this.x<x).getY()<y)){
                    return true;
                } else {
                    return false;
                }
            } else if(this.x == x){
                if(getFigrkaDirectionY(sach, this.y<y)==null){
                    return true;
                } else if(getFigrkaDirectionY(sach, this.y<y).getColor()!=farba
                        && getFigrkaDirectionY(sach, this.y<y).getX()==x
                        && getFigrkaDirectionY(sach, this.y<y).getY()==y
                        && (getFigrkaDirectionY(sach, this.y<y).getType()!=Figurka.Type.kral || !detKral)){
                    return true;
                } else if((this.y<y)?
                        getFigrkaDirectionY(sach, true).getY()>y :
                        getFigrkaDirectionY(sach, false).getY()<y){      
                    return true;
                } else {
                    return false;
                }
            } else if(this.y == y) {
                if(getFigrkaDirectionX(sach, this.x<x)==null){     
                    return true;
                } else if(getFigrkaDirectionX(sach, this.x<x).getColor()!=farba
                        && getFigrkaDirectionX(sach, this.x<x).getX()==x
                        && getFigrkaDirectionX(sach, this.x<x).getY()==y
                        && (getFigrkaDirectionX(sach, this.x<x).getType()!=Figurka.Type.kral || !detKral)){
                    return true;
                } else if((this.x<x)?
                        getFigrkaDirectionX(sach, true).getX()>x :
                        getFigrkaDirectionX(sach, false).getX()<x){
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     * Vracia reťazcovú reprezentáciu danej figúrky
     * @return Reťazec vo formáte "Dam" + "B" ako biela alebo "C" ako čierna
     */
    @Override
    public String toString() {
        return "Dam" + ((farba==Figurka.Color.biela)? "B" : "C");
    } 
}
