package sach.logic;

/**
 * Reprezentuje vlastnosti spoločné pre všetky figúrky
 * @author Adam Netočný
 */
public interface FigurkaInterface {
    /**
     * Kontrola ťahu figúrkou
     * @param x X-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param y Y-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param sach Udáva pole, v ktorom sú rozmiestnené figúrky pričom sa berie v úvahu aj rozmiestnenie ostatných figúriek
     * @return true, ak sa môže figúrka presunúť, inak vráti false
     * @throws Figurka.PesiakNaKonciPola
     * @throws Figurka.Rozsada
     */
    boolean tah(int x, int y, FigurkaInterface[][] sach) throws Figurka.PesiakNaKonciPola, Figurka.Rozsada;
    /**
     * Kontrola ťahu figúrkou
     * @param x X-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param y Y-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param sach Udáva pole, v ktorom sú rozmiestnené figúrky pričom sa berie v úvahu aj rozmiestnenie ostatných figúriek
     * @param detKral Určuje či má, alebo nemá brať v úvahu, že figúrkou, na ktorú sa snaží preisť je kráľ
     * @return true, ak sa môže figúrka presunúť, inak vráti false
     * @throws Figurka.PesiakNaKonciPola 
     * @throws Figurka.Rozsada
     */
    boolean tah(int x, int y, FigurkaInterface[][] sach, boolean detKral) throws Figurka.PesiakNaKonciPola, Figurka.Rozsada;
    /**
     * Vráti x-ovú pozíciu figúrky v rámci poľa
     * @return X-ová pozícia figúrky v rámci poľa
     */
    int getX();
    /**
     * Vráti y-ovú pozíciu figúrky v rámci poľa
     * @return Y-ová pozícia figúrky v rámci poľa
     */
    int getY();
    /**
     * Vráti farbu figúrky
     * @return Farba figúrky
     */
    Figurka.Color getColor();
    /**
     * Vráti typ figúrky
     * @return Typ figúrky
     */
    Figurka.Type getType();
    /**
     * Oznámi figúrke, na akej pozícii sa nachádza
     * @param x X-ová pozícia figúrky v rámci poľa
     * @param y Y-ová pozícia figúrky v rámci poľa
     */
    void setXY(int x, int y);
}
