package sach.logic;

import sach.*;
import sach.logic.Figurka.Color;


/**
 * Uchováva status aktuálnej hry, nie je síce plnohodnotná komponenta(nekomunikuje s ostatnými), ale môžeme ho tak brať keďže sa podiela na tvorbe hry
 * @author Adam Netočný
 */
public class GameStatus implements ComponentInterface {
    /**
     * Kontajner vyhodených figúriek
     */
    private VyhodeneKontajner vyhod;

    /**
     * Farba figúriek hráča na tomto počítači
     */
    private Figurka.Color farba;

    /**
     * Uchováva odkaz na objekt jadra
     */
    private Sachovnica sach;
    
    /**
     * Detekuje, či hra začala
     */
    private boolean gameStarted;

    /**
     * Vytvára nový objekt typu GameStatus
     * @param farba Farba figúriek hráča na tomto počítači
     * @param sach Odkaz na objekt jadra
     */
    public GameStatus(Color farba, Sachovnica sach) {
        this.vyhod = new VyhodeneKontajner();
        this.farba = farba;
        this.gameStarted = false;
        this.sach = sach;
    }
    
    /**
     * Vracia kontajner s vyhodenými figúrkami
     * @return Kontajner s vyhodenými figúrkami
     */
    public VyhodeneKontajner getKontajner(){
        return vyhod;
    }

    /**
     * Detekuje či hra začala
     * @return true - hra začala; false - hra nezačala
     */
    public boolean getGameStarted(){
        return gameStarted;
    }

    /**
     * Vracia klon poľa s rozostavením figúriek
     * @return Klon poľa s rozostavením figúriek
     */
    public FigurkaInterface[][] getSachovnica() {
        return this.sach.getSach();
    }

    /**
     * Spúšťa alebo zastavuje hru
     * @param gameStarted true - začne hru; false - ukončí hru
     */
    public void setGameSterted(boolean gameStarted){
        this.gameStarted = gameStarted;
    }

    /**
     * Nastavuje farbu figúriek hráča na tomto počítači
     * @param farba Farba figúriek hráča na tomto počítači
     */
    public void setPlayerColor(Figurka.Color farba){
        this.farba = farba;
    }

    /**
     * Vracia farbu figúriek hráča na tomto počítači
     * @return Farba figúriek hráča na tomto počítači
     */
    public Figurka.Color getPlayerColor(){
        return this.farba;
    }

    /**
     * Vracia farbu figúriek hráča na tomto počítači
     * @return Farba figúriek hráča na tomto počítači
     */
    public Figurka.Color getOponentColor(){
        return (getPlayerColor() == Figurka.Color.biela)? Color.cierna : Color.biela;
    }

    /**
     * Volaná na komponente pri ukončovaní programu, nemá bližší význam
     */
    public void exit() {
        try {
            finalize();
        } catch (Throwable ex) {
        }
    }
}
