package sach.logic;

/**
 * Implementácia figúrky typu kôň
 * @author Adam Netočný
 */
class Kon extends Figurka implements FigurkaInterface {
    /**
     * Vytvorí novú figúrku typu kôň
     * @param x X-ová pozícia, na ktorej je kôň umiestnený
     * @param y Y-ová pozícia, na ktorej je kôň umiestnený
     * @param farba Farba novovytvorenej figúrky
     */
    Kon(int x, int y, Figurka.Color farba){
        super(x, y, farba);
        typ = Figurka.Type.kon;
    }

    /**
     * Kontrola ťahu figúrkou typu kôň
     * @param x X-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param y Y-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param sach Udáva pole, v ktorom sú rozmiestnené figúrky pričom sa berie v úvahu aj rozmiestnenie ostatných figúriek
     * @param detKral Určuje či má, alebo nemá brať v úvahu, že figúrkou, na ktorú sa snaží preisť je kráľ
     * @return true, ak sa môže figúrka presunúť, inak vráti false
     */
    @Override
    public boolean tah(int x, int y, FigurkaInterface[][] sach, boolean detKral) {
        if((this.x<8 && this.x>=0) && (this.y<8 && this.y>=0) && (x<8 && x>=0) && (y<8 && y>=0)){
            if((Math.abs(this.x-x)==2 && Math.abs(this.y-y)==1)
                    || (Math.abs(this.x-x)==1 && Math.abs(this.y-y)==2)){
                if(sach[x][y]!=null){
                    if(sach[x][y].getColor()!=farba
                            && (sach[x][y].getType()!=Figurka.Type.kral || !detKral)){
                        return true;
                    } else {
                        return false;
                    }
                } else {    
                    return true;
                }
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     * Vracia reťazcovú reprezentáciu danej figúrky
     * @return Reťazec vo formáte "Kon" + "B" ako biely alebo "C" ako čierny
     */
    @Override
    public String toString() {
        return "Kon" + ((farba==Figurka.Color.biela)? "B" : "C");
    } 
}
