package sach.logic;

/**
 * Implementácia figúrky typu kráľ
 * @author Adam Netočný
 */
class Kral extends Figurka implements FigurkaInterface {
    /**
     * Uchováva logickú hodnotu o tom, či už kráľ urobil prvý ťah
     */
    private boolean prvyTah;

    /**
     * Uchováva logickú hodnotu o tom, či už bol kráľ v šachu
     */
    private boolean bolVSachu;
    
    /**
     * Vytvorí novú figúrku typu kôň
     * @param x X-ová pozícia, na ktorej je kráľ umiestnený
     * @param y Y-ová pozícia, na ktorej je kráľ umiestnený
     * @param farba Farba novovytvorenej figúrky
     */
    Kral(int x, int y, Figurka.Color farba){
        super(x, y, farba);
        typ = Figurka.Type.kral;
        prvyTah = true;
        bolVSachu = false;
    }

    /**
     * Kontrola ťahu figúrkou typu kráľ
     * @param x X-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param y Y-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param sach Udáva pole, v ktorom sú rozmiestnené figúrky pričom sa berie v úvahu aj rozmiestnenie ostatných figúriek
     * @param detKral Určuje či má, alebo nemá brať v úvahu, že figúrkou, na ktorú sa snaží preisť je kráľ
     * @return true, ak sa môže figúrka presunúť, inak vráti false
     * @throws sach.Figurka.Rozsada Vyvolá sa, ak užívateľ požaduje rozšádu
     */
    @Override
    public boolean tah(int x, int y, FigurkaInterface[][] sach, boolean detKral) throws Rozsada {
        if((this.x<8 && this.x>=0) && (this.y<8 && this.y>=0) && (x<8 && x>=0) && (y<8 && y>=0)){
            if((Math.abs(this.x-x)== 0 || Math.abs(this.x-x)== 1)
                    && (Math.abs(this.y-y)== 0 || Math.abs(this.y-y)== 1)){

                if(sach[x][y]==null){
                    return detekujKrala(x, y, sach);
                } else {
                    if(sach[x][y].getColor()!=farba
                            && (sach[x][y].getType()!=Figurka.Type.kral || !detKral)){
                        return detekujKrala(x, y, sach);
                    } else {
                        return false;
                    }
                }
            } else if(sach[x][y]!=null && prvyTah && !bolVSachu){
                if(sach[x][y].getType() == Figurka.Type.veza && sach[x][y].getColor()==farba){
                    if(((Veza)sach[x][y]).getPrvyTah()){
                        if(getFigrkaDirectionY(sach, this.y<y)==sach[x][y]){
                            if(detekujKrala(x, y, sach)){
                                throw new Rozsada(this);
                            } else {
                                return false;
                            }
                        } else {
                            return false;
                        }
                    } else {
                        return false;
                    }
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     * Detekuje, či kráľ nebude po vykonaní ťahu v kolízii s druhým kráľom
     * @param x X-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param y Y-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param sach Udáva pole, v ktorom sú rozmiestnené figúrky
     * @return true - ak nebude v kolízii, inak false
     */
    private boolean detekujKrala(int x, int y, FigurkaInterface[][] sach){
        for(int a = -1; a < 2; a++){
            for(int b = -1; b < 2; b++){
                if((x+a<8 && x+a>=0) && (y+b<8 && y+b>=0) && (a!=0 || b!=0))
                    if(sach[x+a][y+b]!=null && sach[x+a][y+b]!=this){
                        if(sach[x+a][y+b].getType()==Figurka.Type.kral){
                            return false;
                        }
                    }
            }
        }
        prvyTah = false;
 
        return true;
    }

    /**
     * Nastavuje logickú hodnotu, či kráľ urobil prvý ťah
     * @param tah Reprezentuje logickú hodnotu o tom, či už kráľ urobil prvý ťah
     */
    public void setPrvyTah(boolean tah){
        prvyTah = tah;
    }

    /**
     * Nastavuje logickú hodnotu o tom, či už bol kráľ v šachu
     * @param sch Reprezentuje logickú hodnotu o tom, či už bol kráľ v šachu
     */
    public void setBolVSachu(boolean sch){
        bolVSachu = sch;
    }

    /**
     * Vracia reťazcovú reprezentáciu danej figúrky
     * @return Reťazec vo formáte "Krl" + "B" ako biely alebo "C" ako čierny
     */
    @Override
    public String toString() {
        return "Krl" + ((farba==Figurka.Color.biela)? "B" : "C");
    } 
}
