package sach.logic;

/**
 * Implementácia figúrky typu pešiak
 * @author Adam Netočný
 */
class Pesiak extends Figurka implements FigurkaInterface {
    /**
     * Uchováva logickú hodnotu o tom, či už pešiak urobil prvý ťah
     */
    private boolean prvyTah;

    /**
     * Uchováva logickú hodnotu o tom, ktorým smerom sa má pešiak pohybovať (hore/dole)
     */
    private boolean smer;

    /**
     * Vytvorí novú figúrku typu pešiak
     * @param x X-ová pozícia, na ktorej je pešiak umiestnený
     * @param y Y-ová pozícia, na ktorej je pešiak umiestnený
     * @param farba Farba novovytvorenej figúrky
     */
    Pesiak(int x, int y, Figurka.Color farba){
        super(x, y, farba);
        prvyTah = true;

        if(this.x > 3){
            smer = true;
        } else {
            smer = false;
        }

        typ = Figurka.Type.pesiak;
    }

    /**
     * Kontrola ťahu figúrkou typu pešiak
     * @param x X-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param y Y-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param sach Udáva pole, v ktorom sú rozmiestnené figúrky pričom sa berie v úvahu aj rozmiestnenie ostatných figúriek
     * @param detKral Určuje či má, alebo nemá brať v úvahu, že figúrkou, na ktorú sa snaží preisť je kráľ
     * @return true, ak sa môže figúrka presunúť, inak vráti false
     * @throws sach.Figurka.PesiakNaKonciPola Vyvolá sa, ak pešiak dôjde na koniec poľa
     */
    @Override
    public boolean tah(int x, int y, FigurkaInterface[][] sach, boolean detKral) throws PesiakNaKonciPola {
        if((this.x<8 && this.x>=0) && (this.y<8 && this.y>=0) && (x<8 && x>=0) && (y<8 && y>=0)){
            //urcenie smeru pohybu
            if(smer && this.x < x){
                return false;
            } else if(!smer && this.x > x){
                return false;
            }

            //pohyb dopredu
            if((this.y == y && (Math.abs(this.x-x)==1
                    || (prvyTah && Math.abs(this.x-x)==2))) && detKral){
                if(Math.abs(this.x-x)==2) {
                    if(smer){
                        if(sach[this.x-1][y]!=null)
                            return false;
                    } else {
                        if(sach[this.x+1][y]!=null)
                            return false;
                    }
                }                

                if(sach[x][y]==null){
                    if((smer && x==0) || (!smer && x==7)){
                        prvyTah = false;
                        throw new PesiakNaKonciPola(this);
                    }
                    prvyTah = false;
                    return true;
                } else {
                    return false;
                }
            //pohyb do stran
            } else if(Math.abs(this.x-x)==1 && Math.abs(this.y-y)==1) {
                if(sach[x][y]!=null){
                    if(sach[x][y].getColor()!=farba && (sach[x][y].getType()!=Figurka.Type.kral || !detKral)){
                        if((smer && x==0) || (!smer && x==7)){
                            prvyTah = false;
                            throw new PesiakNaKonciPola(this);
                        }
                        prvyTah = false;
                        return true;
                    } else {
                        return false;
                    }
                } else if(!detKral) {
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     * Vracia reťazcovú reprezentáciu danej figúrky
     * @return Reťazec vo formáte "Pes" + "B" ako biely alebo "C" ako čierny
     */
    @Override
    public String toString() {
        return "Pes" + ((farba==Figurka.Color.biela)? "B" : "C");
    } 
}
