package sach.logic;

/**
 * Reprezntuje nosiča informácii o bode hracieho poľa
 * @author Adam Netočný
 */
public class Point {
    /**
     * X-ová pozícia políčka
     */
    private int x;
    /**
     * Y-ová pozícia políčka
     */
    private int y;
    
    /**
     * Vytvorí objek s danou x a y súradnicou
     * @param x X-ová pozícia políčka
     * @param y Y-ová pozícia políčka
     */
    public Point(int x, int y){
        this.x = x;
        this.y = y;
        check();
    }

    /**
     * Vytvorí objek podľa jednoduchého číselného indexu políčka
     * @param index Číselné označovanie políčka(0-63)
     */
    public Point(int index){
        this.x = index / 8;
        this.y = index % 8;
        check();
    }

    /**
     * Odkóduje šachové označovanie a vytvorí nový objekt
     * @param index Šachové označovanie políčka (napr. A8(A1-H8))
     */
    public Point(String index){
        String[] pismeno = {"A", "B", "C", "D", "E", "F", "G", "H"};
        int posY = 0;
        for(int a = 0; a < pismeno.length; a++){
            if(pismeno[a].equals(index.charAt(0) + "")){
                posY = a;
                break;
            }
        }
        int posX = 8-Integer.parseInt(index.charAt(1) + "");
        this.x = posX;
        this.y = posY;
        check();
    }
    
    /**
     * Zakóduje bod na šachové označovanie
     * @return Šachové označovanie políčka
     */
    @Override
    public String toString(){
        String[] pismeno = {"A", "B", "C", "D", "E", "F", "G", "H"};
        return pismeno[y] + (8-x);
    }

    /**
     * Kontroluje rozsah príjmaných hodnôt(0-7)
     */
    private void check(){
        if(x >= 7) {
            x = 7;
        }
        if(x < 0) {
            x = 0;
        }
        if(y >= 7) {
            y = 7;
        }
        if(y < 0) {
            y = 0;
        }
    }

    /**
     * Vráti X-ovú pozíciu políčka
     * @return X-ová pozícia políčka
     */
    public int getX() {
        return x;
    }

    /**
     * Vráti Y-ovú pozíciu políčka
     * @return Y-ová pozícia políčka
     */
    public int getY() {
        return y;
    }

    /**
     * Vráti pozíciu políčka vrámci jednoduchého číselného indexu(0-63)
     * @return Jednoduchý číselnéhý index políčka
     */
    public int getIndex(){
        return (8*x)+y;
    }

    /**
     * Nastaví X-ovú pozíciu políčka
     * @param x X-ová pozícia políčka
     */
    public void setX(int x) {
        this.x = x;
        check();
    }

    /**
     * Nastaví Y-ovú pozíciu políčka
     * @param y Y-ová pozícia políčka
     */
    public void setY(int y) {
        this.y = y;
        check();
    }

    /**
     * Porovnáva dva body pomocou hashCode()
     * @param obj Bod, z ktorým chceme porovnať
     * @return true - zhodné; false - rozdielne
     */
    @Override
    public boolean equals(Object obj) {
        if (obj.getClass() == this.getClass()) {
            return (x == ((Point)obj).getX() && y == ((Point)obj).getY());
        } else {
            return false;
        }
    }

    /**
     * Prekrytie metódy hashCode(rozoznáva dvojice napr. 3, 1 a 1, 3)
     * @return "hash"
     */
    @Override
    public int hashCode() {
        return ((17*x)+(15*y));
    }
}
