package sach.logic;

/**
 * Sprostretkuváva detekciu šachu a matu
 * @author Adam Netočný
 */
class SachMatDetektor {
    //<editor-fold defaultstate="collapsed" desc="Detekcia sachu a matu">
    /**
     * Detekuje či nie je jeden z kráľov v šachu alebo nedostal mat
     * @param kr Odkaz na kráľa
     * @param sach Štvorcové pole figúriek, zaznamenáva aktuálne rozloženie figúriek na hraciej ploche
     * @return true - Kráľ dostal mat; inak false
     */
    public static boolean detekujSachMat(Kral kr, FigurkaInterface[][] sach) {
        if(detekujSach(kr, sach)){
            if(detekujMat(kr, sach)){
                return true;
            }
            kr.setBolVSachu(true);
        }
        return false;
    }


    //<editor-fold defaultstate="collapsed" desc="Detekcia sachu">
    /**
     * Detekuje či je kráľ v šachu
     * @param kr Odkaz na kráľa, ktorého chceme testovať
     * @param sach Štvorcové pole figúriek, zaznamenáva aktuálne rozloženie figúriek na hraciej ploche
     * @return true - je v šachu; false - nie je
     */
    public static boolean detekujSach(Kral kr, FigurkaInterface[][] sach){
        return detekujSach(kr, sach, 0, 0);
    }
    
    /**
     * Detekuje či je kráľ v šachu vzhľadom k relatívnym súradniciam
     * @param kr Odkaz na kráľa, ktorého chceme testovať
     * @param sach Štvorcové pole figúriek, zaznamenáva aktuálne rozloženie figúriek na hraciej ploche
     * @param posX X-ové posunutie
     * @param posY Y-ové posunutie
     * @return true - je na danej pozícii v šachu; false - nie je
     */
    public static boolean detekujSach(Kral kr, FigurkaInterface[][] sach, int posX, int posY){
        sach[kr.getX()][kr.getY()] = null;
        for (int a = 0; a < 8; a++){
            for (int b = 0; b < 8; b++){
                try {
                    if(sach[a][b]!=null){
                        if(sach[a][b] != kr && sach[a][b].getType() != Figurka.Type.kral && sach[a][b].getColor() != kr.getColor()){
                            if(sach[a][b].tah(kr.getX()+posX, kr.getY()+posY, sach, false)){
                                sach[kr.getX()][kr.getY()] = kr;
                                return true;
                            }
                        }
                    }
                } catch (Throwable ex) { }
            }
        }
        sach[kr.getX()][kr.getY()] = kr;
        return false;
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="Detekcia matu">
    /**
     * Detekuje či kráľ dostal mat, pričom sa predpokladá, že stojí v šachu
     * @param kr Odkaz na kráľa, ktorého chceme testovať
     * @param sach Štvorcové pole figúriek, zaznamenáva aktuálne rozloženie figúriek na hraciej ploche
     * @return true - dostal mat; false - nedostal
     */
    public static boolean detekujMat(Kral kr, FigurkaInterface[][] sach){
        int x = kr.getX();
        int y = kr.getY();
        boolean mat = true;
        //<editor-fold defaultstate="collapsed" desc="Test okolitych policok">
        for(int a = -1; a < 2; a++){
            for(int b = -1; b < 2; b++){
                if((x+a<8 && x+a>=0) && (y+b<8 && y+b>=0) && (a!=0 || b!=0))
                    if(sach[x+a][y+b]==null){
                        if(!detekujSach(kr, sach, a, b)){
                            if(detekujKrala(kr, sach, a, b)){
                                mat = false;
                                break;
                            }
                        }
                    }
            }
            if(!mat)
                break;
        }
        //</editor-fold>

        if(mat){
            //<editor-fold defaultstate="collapsed" desc="Zisti, ktora figurka ho ohrozuje">
            FigurkaInterface figurka = null;
            for (int a = 0; a < 8; a++){
                for (int b = 0; b < 8; b++){
                    try {
                        if(sach[a][b]!=null){
                            if(sach[a][b] != kr && sach[a][b].getType() != Figurka.Type.kral && sach[a][b].getColor() != kr.getColor()){
                                if(sach[a][b].tah(kr.getX(), kr.getY(), sach, false)){
                                    figurka = sach[a][b];
                                }
                            }
                        }
                    } catch (Throwable ex) { }
                }
            }
            //</editor-fold>

            if(figurka==null){
                return mat;
            }
            //<editor-fold defaultstate="collapsed" desc="Zisti, ci nemozno danu figurku vziat">
            for (int a = 0; a < 8; a++){
                for (int b = 0; b < 8; b++){
                    try {
                        if(sach[a][b]!=null){
                            if(sach[a][b] != figurka
                                && sach[a][b].getColor() == kr.getColor()){
                                if(sach[a][b].tah(figurka.getX(), figurka.getY(), sach, false)){
                                    if(sach[a][b].getType() == Figurka.Type.kral){
                                        sach[figurka.getX()][figurka.getY()] = null;
                                        if(!detekujSach((Kral)sach[a][b], sach,
                                                figurka.getX()-sach[a][b].getX(),
                                                figurka.getY()-sach[a][b].getY())){
                                            mat = false;
                                        }
                                        sach[figurka.getX()][figurka.getY()] = figurka;
                                    } else {
                                        mat = false;
                                    }
                                }
                            }
                        }
                    } catch (Throwable ex) { }
                }
            }
            //</editor-fold>

            if(mat){
                //<editor-fold defaultstate="collapsed" desc="Zisti ci nemozno danu figurku predkryt">
                if(figurka!=null){
                    if(!(figurka.getType() == Figurka.Type.kon || figurka.getType() == Figurka.Type.pesiak)){
                        //ostatne mozeme este predkryt
                        //<editor-fold defaultstate="collapsed" desc="Ak je figurka veza">
                        if(figurka.getType() == Figurka.Type.veza){
                            if(figurka.getX()==kr.getX()){
                                int smr = 0;
                                if(figurka.getY()<kr.getY()){
                                    smr = 1;
                                } else {
                                    smr = -1;
                                }

                                int pos = figurka.getY()+smr;

                                for(; pos<8 && pos>=0 && pos != kr.getY(); pos+=smr){
                                    for (int a = 0; a < 8; a++){
                                        for (int b = 0; b < 8; b++){
                                            try {
                                                if(sach[a][b]!=null){
                                                    if(sach[a][b] != figurka
                                                        && sach[a][b].getType() != Figurka.Type.kral
                                                        && sach[a][b].getColor() == kr.getColor()){
                                                        if(sach[a][b].tah(figurka.getX(), pos, sach, false)){
                                                            mat = false;
                                                            break;
                                                        }
                                                    }
                                                }
                                            } catch (Throwable ex) { }
                                        }
                                        if(!mat)
                                            break;
                                    }
                                    if(!mat)
                                        break;
                                }
                            } else if(figurka.getY() == kr.getY()){
                                int smr = 0;
                                if(figurka.getX()<kr.getX()){
                                    smr = 1;
                                } else {
                                    smr = -1;
                                }

                                int pos = figurka.getX()+smr;

                                for(; pos<8 && pos>=0 && pos!=kr.getX(); pos+=smr){
                                    for (int a = 0; a < 8; a++){
                                        for (int b = 0; b < 8; b++){
                                            try {
                                                if(sach[a][b]!=null){
                                                    if(sach[a][b] != figurka
                                                        && sach[a][b].getType() != Figurka.Type.kral
                                                        && sach[a][b].getColor() == kr.getColor()){
                                                        if(sach[a][b].tah(pos, figurka.getY(), sach, false)){
                                                            mat = false;
                                                            break;
                                                        }
                                                    }
                                                }
                                            } catch (Throwable ex) { }
                                        }
                                        if(!mat)
                                            break;
                                    }
                                    if(!mat)
                                        break;
                                }
                            }
                        //</editor-fold>

                        //<editor-fold defaultstate="collapsed" desc="Ak je dana figurka strelec">
                        } else if(figurka.getType() == Figurka.Type.strelec){
                            int xplus = 0;
                            int yplus = 0;

                            boolean doprava = figurka.getY()<kr.getY();
                            boolean dole = figurka.getX()<kr.getX();
                            if(doprava && dole){
                                xplus = 1;
                                yplus = 1;
                            } else if(doprava && !dole){
                                xplus = -1;
                                yplus = 1;
                            } else if(!doprava && dole){
                                xplus = 1;
                                yplus = -1;
                            } else if(!doprava && !dole){
                                xplus = -1;
                                yplus = -1;
                            }

                            int a = figurka.getY()+yplus;
                            int b = figurka.getX()+xplus;
                            while((a<=7 && a>=0) && (b<=7 && b>=0) && (a != kr.getY() && b != kr.getX())){
                                for (int posx = 0; posx < 8; posx++){
                                    for (int posy = 0; posy < 8; posy++){
                                        try {
                                            if(sach[posx][posy]!=null){
                                                if(sach[posx][posy] != figurka
                                                    && sach[posx][posy].getType() != Figurka.Type.kral
                                                    && sach[posx][posy].getColor() == kr.getColor()){
                                                    if(sach[posx][posy].tah(b, a, sach, false)){
                                                        mat = false;
                                                        break;
                                                    }
                                                }
                                            }
                                        } catch (Throwable ex) { }
                                    }
                                    if(!mat)
                                        break;
                                }
                                if(!mat)
                                    break;
                                a+=yplus;
                                b+=xplus;
                            }
                        //</editor-fold>

                        //<editor-fold defaultstate="collapsed" desc="Ak je dana figurka dama">
                        } else if(figurka.getType() == Figurka.Type.dama) {
                            if(figurka.getX()==kr.getX()){
                                int smr = 0;
                                if(figurka.getY()<kr.getY()){
                                    smr = 1;
                                } else {
                                    smr = -1;
                                }

                                int pos = figurka.getY()+smr;

                                for(; pos<8 && pos>=0 && pos != kr.getY(); pos+=smr){
                                    for (int a = 0; a < 8; a++){
                                        for (int b = 0; b < 8; b++){
                                            try {
                                                if(sach[a][b]!=null){
                                                    if(sach[a][b] != figurka
                                                        && sach[a][b].getType() != Figurka.Type.kral
                                                        && sach[a][b].getColor() == kr.getColor()){
                                                        if(sach[a][b].tah(figurka.getX(), pos, sach, false)){
                                                            mat = false;
                                                            break;
                                                        }
                                                    }
                                                }
                                            } catch (Throwable ex) { }
                                        }
                                        if(!mat)
                                            break;
                                    }
                                    if(!mat)
                                        break;
                                }
                            } else if(figurka.getY() == kr.getY()){
                                int smr = 0;
                                if(figurka.getX()<kr.getX()){
                                    smr = 1;
                                } else {
                                    smr = -1;
                                }

                                int pos = figurka.getX()+smr;

                                for(; pos<8 && pos>=0 && pos != kr.getX(); pos+=smr){
                                    for (int a = 0; a < 8; a++){
                                        for (int b = 0; b < 8; b++){
                                            try {
                                                if(sach[a][b]!=null){
                                                    if(sach[a][b] != figurka
                                                        && sach[a][b].getType() != Figurka.Type.kral
                                                        && sach[a][b].getColor() == kr.getColor()){
                                                        if(sach[a][b].tah(pos, figurka.getY(), sach, false)){
                                                            mat = false;
                                                            break;
                                                        }
                                                    }
                                                }
                                            } catch (Throwable ex) { }
                                        }
                                        if(!mat)
                                            break;
                                    }
                                    if(!mat)
                                        break;
                                }
                            } else {
                                int xplus = 0;
                                int yplus = 0;

                                boolean doprava = figurka.getY()<kr.getY();
                                boolean dole = figurka.getX()<kr.getX();
                                if(doprava && dole){
                                    xplus = 1;
                                    yplus = 1;
                                } else if(doprava && !dole){
                                    xplus = -1;
                                    yplus = 1;
                                } else if(!doprava && dole){
                                    xplus = 1;
                                    yplus = -1;
                                } else if(!doprava && !dole){
                                    xplus = -1;
                                    yplus = -1;
                                }

                                int a = figurka.getY()+yplus;
                                int b = figurka.getX()+xplus;
                                while((a<=7 && a>=0) && (b<=7 && b>=0) & (a != kr.getY() && b != kr.getX())){
                                    for (int posx = 0; posx < 8; posx++){
                                        for (int posy = 0; posy < 8; posy++){
                                            try {
                                                if(sach[posx][posy]!=null){
                                                    if(sach[posx][posy] != figurka
                                                        && sach[posx][posy].getType() != Figurka.Type.kral
                                                        && sach[posx][posy].getColor() == kr.getColor()){
                                                        if(sach[posx][posy].tah(b, a, sach, false)){
                                                            mat = false;
                                                            break;
                                                        }
                                                    }
                                                }
                                            } catch (Throwable ex) { }
                                        }
                                        if(!mat)
                                            break;
                                    }
                                    if(!mat)
                                        break;
                                    a+=yplus;
                                    b+=xplus;
                                }
                            }
                        }
                        //</editor-fold>
                    }
                }
                //</editor-fold>
            }
        }
        return mat;
    }

    /**
     * Detekuje, či kráľ nebude v kolízii s druhým kráľom vzhľadom k relatívnym súradniciam
     * @param kr Odkaz na kráľa, ktorého chceme testovať
     * @param sach Udáva pole, v ktorom sú rozmiestnené figúrky
     * @param posX X-ové posunutie
     * @param posY Y-ové posunutie
     * @return true - ak nebude v kolízii, inak false
     */
    private static boolean detekujKrala(Kral kr, FigurkaInterface[][] sach, int posX, int posY) {
        int x = kr.getX()+posX;
        int y = kr.getY()+posY;
        sach[kr.getX()][kr.getY()] = null;
        for(int a = -1; a < 2; a++){
            for(int b = -1; b < 2; b++){
                if((x+a<8 && x+a>=0) && (y+b<8 && y+b>=0) && (a!=0 || b!=0))
                    if(sach[x+a][y+b]!=null && sach[x+a][y+b]!=kr){
                        if(sach[x+a][y+b].getType()==Figurka.Type.kral){
                            sach[kr.getX()][kr.getY()] = kr;
                            return false;
                        }
                    }
            }
        }

        sach[kr.getX()][kr.getY()] = kr;
        return true;
    }
    //</editor-fold>

    //</editor-fold>
}
