package sach.logic;

import sach.*;
import sach.eventmodel.*;
import sach.logic.Figurka.*;
import sach.network.NetworkGame.*;

/**
 * Samotný Engine hry, kombinuje model figúriek a sieťové rozhrannie
 * @author Adam Netočný
 */
public class Sachovnica extends TahHandler implements ComponentInterface {
    /**
     * Štvorcové pole figúriek, zaznamenáva aktuálne rozloženie figúriek na hraciej ploche
     */
    private FigurkaInterface[][] sach;

    /**
     * Odkaz na čierného kráľa
     */
    private Kral ckr;
    /**
     * Odkaz na bielého kráľa
     */
    private Kral bkr;

    /**
     * Ukladá typ figúrky ak bol nejaký vybraný
     */
    private Figurka.Type vybrana;

    /**
     * Farba figúriek hráča na tahu
     */
    private Figurka.Color farba = Color.biela;

    /**
     * Logická hodnota, ktorá detekuje, či šachovnica očakáva zvolenie figúrky, alebo nie
     */
    private boolean ocakava = false;

    /**
     * Volaná pri prijatí události typu ťah
     * @param event Daná udalosť
     */
    @Override
    public void tah(TahEvent event) {
        if(!event.equals(this.event)){
            checkTah(event.getOdkial().getX(), event.getOdkial().getY(), event.getKam().getX(), event.getKam().getY());
        }
    }

    /**
     * Volaná pri prijatí události typu chybný ťah
     * @param event Daná udalosť
     */
    @Override
    public void chybyTah(TahEvent event) {
        if(!event.equals(this.event)){
        }
    }

    /**
     * Volaná pri prijatí události typu výber figúrky
     * @param event Daná udalosť
     */
    @Override
    public void vyberFigurku(TahEvent event) {
        if(event.getTypFig() != null) {
            vybrana = event.getTypFig();
            if(ocakava)
                fireVyberFigurku(event.getOdkial(), event.getKam(), event.getTypFig(), event.getFarba());
                figurkaVybrana(event.getOdkial().getX(), event.getOdkial().getY(), event.getKam().getX(), event.getKam().getY());
                return;
        }
        fireVyberFigurku(event.getOdkial(), event.getKam(), event.getTypFig(), event.getFarba());
    }

    /**
     * Volaná pri prijatí události typu chybová hláška
     * @param event Daná udalosť
     */
    @Override
    public void chyba(TahEvent event) {
        if(!event.equals(this.event)){
            fireChyba(event.getChyba());
        }
    }

    /**
     * Volaná pri prijatí události typu šach
     * @param event Daná udalosť
     */
    @Override
    public void sach(TahEvent event) {
        
    }

    /**
     * Volaná pri prijatí události typu mat
     * @param event Daná udalosť
     */
    @Override
    public void mat(TahEvent event) {
        
    }

    //<editor-fold defaultstate="collapsed" desc="Konstruktor(vytvorenie sachovnice) a destruktor(zrusenie pripojenia)">
    /**
     * Vytvorí nový herný Engine spolu s rozostavením figúriek
     */
    public Sachovnica(){        
        sach = new FigurkaInterface[8][8];
        sach[0][0] = new Veza(0, 0, Figurka.Color.cierna);
        sach[0][1] = new Kon(0, 1, Figurka.Color.cierna);
        sach[0][2] = new Strelec(0, 2, Figurka.Color.cierna);
        sach[0][3] = new Dama(0, 3, Figurka.Color.cierna);
        ckr = new Kral(0, 4, Figurka.Color.cierna);
        sach[0][4] = ckr;
        sach[0][5] = new Strelec(0, 5, Figurka.Color.cierna);
        sach[0][6] = new Kon(0, 6, Figurka.Color.cierna);
        sach[0][7] = new Veza(0, 7, Figurka.Color.cierna);

        sach[1][0] = new Pesiak(1, 0, Figurka.Color.cierna);
        sach[1][1] = new Pesiak(1, 1, Figurka.Color.cierna);
        sach[1][2] = new Pesiak(1, 2, Figurka.Color.cierna);
        sach[1][3] = new Pesiak(1, 3, Figurka.Color.cierna);
        sach[1][4] = new Pesiak(1, 4, Figurka.Color.cierna);
        sach[1][5] = new Pesiak(1, 5, Figurka.Color.cierna);
        sach[1][6] = new Pesiak(1, 6, Figurka.Color.cierna);
        sach[1][7] = new Pesiak(1, 7, Figurka.Color.cierna);

        sach[7][0] = new Veza(7, 0, Figurka.Color.biela);
        sach[7][1] = new Kon(7, 1, Figurka.Color.biela);
        sach[7][2] = new Strelec(7, 2, Figurka.Color.biela);
        sach[7][3] = new Dama(7, 3, Figurka.Color.biela);
        bkr = new Kral(7, 4, Figurka.Color.biela);
        sach[7][4] = bkr;
        sach[7][5] = new Strelec(7, 5, Figurka.Color.biela);
        sach[7][6] = new Kon(7, 6, Figurka.Color.biela);
        sach[7][7] = new Veza(7, 7, Figurka.Color.biela);

        sach[6][0] = new Pesiak(6, 0, Figurka.Color.biela);
        sach[6][1] = new Pesiak(6, 1, Figurka.Color.biela);
        sach[6][2] = new Pesiak(6, 2, Figurka.Color.biela);
        sach[6][3] = new Pesiak(6, 3, Figurka.Color.biela);
        sach[6][4] = new Pesiak(6, 4, Figurka.Color.biela);
        sach[6][5] = new Pesiak(6, 5, Figurka.Color.biela);
        sach[6][6] = new Pesiak(6, 6, Figurka.Color.biela);
        sach[6][7] = new Pesiak(6, 7, Figurka.Color.biela);
    }
    //</editor-fold>
    
    /**
     * Sprostretkuváva samotný ťah s ohľadom na rozšady, pešiakov, šach a mat
     * @param posX X-ová pozícia figúrky, ktorou chceme tiahnuť
     * @param posY y-ová pozícia figúrky, ktorou chceme tiahnuť
     * @param x X-ová pozícia miesta, na ktoré chceme tiahnuť
     * @param y Y-ová pozícia miesta, na ktoré chceme tiahnuť
     */
    private void checkTah(int posX, int posY, int x, int y){
        boolean tah = false;
        if(sach[posX][posY] != null && (posX != x || posY != y)){
            try {
                tah = sach[posX][posY].tah(x, y, sach);
                if (tah) {
                    //<editor-fold defaultstate="collapsed" desc="Bezny tah aj s odstranenim figurky">
                    FigurkaInterface fg = null;
                    if(sach[x][y] != null){
                        fg = sach[x][y];

                    }
                    sach[x][y] = sach[posX][posY];
                    sach[x][y].setXY(x, y);
                    sach[posX][posY] = null;

                    if(SachMatDetektor.detekujSach((getPlayerColor()==Figurka.Color.biela)? bkr : ckr, sach)){
                        sach[posX][posY] = sach[x][y];
                        sach[posX][posY].setXY(posX, posY);
                        sach[x][y] = fg;
                        fireChybnyTah(new Point(posX, posY), new Point(x, y), getPlayerColor());
                    } else {
                        if(fg!=null){
                            SachApp.gameStatus.getKontajner().vloz(fg);
                        }
                        sach[x][y].setXY(x, y);
                        makeMove(posX, posY, x, y);
                    }
                    //</editor-fold>
                } else {
                    fireChybnyTah(new Point(posX, posY), new Point(x, y), getPlayerColor());
                }
            } catch (PesiakNaKonciPola ex) {
                //kontroluje ci nenastane sach aby zbytocne neziadal o typ figurky
                FigurkaInterface vyhodFG = sach[x][y];

                FigurkaInterface fg = sach[posX][posY];
                sach[posX][posY]=null;
                sach[x][y] = new Kon(x, y, getPlayerColor());

                if(SachMatDetektor.detekujSach((getPlayerColor()==Figurka.Color.biela)? bkr : ckr, sach)){
                    sach[posX][posY] = fg;
                    sach[x][y] = vyhodFG;
                    vybrana = null;
                    fireChybnyTah(new Point(posX, posY), new Point(x, y), farba);
                    return;
                } else {
                    sach[posX][posY] = fg;
                    sach[x][y] = vyhodFG;
                    vybrana = null;
                }

                //sach nenastane takze pokracuje ziadostou alebo zamenou
                if(vybrana == null) {
                    ocakava = true;
                    fireVyberFigurku(new Point(posX, posY), new Point(x, y), null, getPlayerColor());
                } else {
                    figurkaVybrana(posX, posY, x, y);
                }
            } catch (Rozsada ex){
                //<editor-fold defaultstate="collapsed" desc="Rozsada">
                if(SachMatDetektor.detekujSach((getPlayerColor()==Figurka.Color.biela)? bkr : ckr, sach)){
                    fireChybnyTah(new Point(posX, posY), new Point(x, y), getPlayerColor());
                    return;
                }
                if(SachMatDetektor.detekujSach((getPlayerColor()==Figurka.Color.biela)? ckr : bkr, sach)){
                    fireChybnyTah(new Point(posX, posY), new Point(x, y), getPlayerColor());
                    return;
                }
                FigurkaInterface fg = sach[posX][posY];
                boolean velka = true;
                if(Math.abs(posY-y)==3){
                    velka = false;
                    sach[posX][posY+1] = sach[x][y];
                    sach[x][y-1] = fg;
                    sach[posX][posY+1].setXY(posX, posY+1);
                    sach[x][y-1].setXY(x, y-1);
                    sach[x][y] = null;
                    sach[posX][posY] = null;
                } else if(Math.abs(posY-y)==4){
                    sach[posX][posY-1] = sach[x][y];
                    sach[x][y+2] = fg;
                    sach[posX][posY-1].setXY(posX, posY-1);
                    sach[x][y+2].setXY(x, y+2);
                    sach[x][y] = null;
                    sach[posX][posY] = null;
                }

                if(SachMatDetektor.detekujSach((getPlayerColor()==Figurka.Color.biela)? bkr : ckr, sach)){
                    if(velka){
                        sach[x][y] = sach[posX][posY-1];
                        sach[posX][posY] = sach[x][y+2];
                        sach[posX][posY].setXY(posX, posY);
                        sach[x][y].setXY(x, y);

                        sach[x][y+2] = null;
                        sach[posX][posY-1] = null;
                    } else {
                        sach[x][y] = sach[posX][posY+1];
                        sach[posX][posY] = sach[x][y-1];
                        sach[posX][posY].setXY(posX, posY);
                        sach[x][y].setXY(x, y);
                        sach[posX][posY+1] = null;
                        sach[x][y-1] = null;
                    }
                    ((Kral)sach[posX][posY]).setPrvyTah(true);
                    fireChybnyTah(new Point(posX, posY), new Point(x, y), getPlayerColor());
                } else {
                    makeMove(posX, posY, x, y);
                }
                //</editor-fold>
            }
        }
    }

    /**
     * Sprostretkuváva komunikáciu medzi komponentami(rozosiela ťahy) pri korektnom ťahu
     * @param posX X-ová pozícia figúrky, ktorou chceme tiahnuť
     * @param posY Y-ová pozícia figúrky, ktorou chceme tiahnuť
     * @param x X-ová pozícia miesta, na ktoré chceme tiahnuť
     * @param y Y-ová pozícia miesta, na ktoré chceme tiahnuť
     */
    private void makeMove(int posX, int posY, int x, int y){
        fireTah(new Point(posX, posY), new Point(x, y), getPlayerColor());
        farba = getOponentColor();
        if(SachMatDetektor.detekujSachMat(bkr, sach)){
            fireMat(Color.biela);
        } else if(SachMatDetektor.detekujSach(bkr, sach)){
            fireSach(Color.biela);
        }
        if(SachMatDetektor.detekujSachMat(ckr, sach)){
            fireMat(Figurka.Color.cierna);
        } else if(SachMatDetektor.detekujSach(ckr, sach)){
            fireSach(Color.cierna);
        }        
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="Pesiak dosiahne koniec pola">
    /**
     * Vykoná zámenu figúrky, ak pešiak dôjde na koniec poľa
     * @param posX X-ová pozícia figúrky, ktorou chceme tiahnuť
     * @param posY Y-ová pozícia figúrky, ktorou chceme tiahnuť
     * @param x X-ová pozícia miesta, na ktoré chceme tiahnuť
     * @param y Y-ová pozícia miesta, na ktoré chceme tiahnuť
     */
    private void figurkaVybrana(int posX, int posY, int x, int y){
        ocakava = false;
        Figurka.Type typ = vybrana;
        Figurka.Color farbaFigurky = farba;

        FigurkaInterface vyhodFG = sach[x][y];

        FigurkaInterface fg = sach[posX][posY];
        sach[posX][posY]=null;
        if(typ == Figurka.Type.kon){
            sach[x][y] = new Kon(x, y, farbaFigurky);
        } else if(typ == Figurka.Type.veza){
            sach[x][y] = new Veza(x, y, farbaFigurky);
        } else if(typ == Figurka.Type.strelec){
            sach[x][y] = new Strelec(x, y, farbaFigurky);
        } else if(typ == Figurka.Type.dama){
            sach[x][y] = new Dama(x, y, farbaFigurky);
        } else {
            sach[posX][posY] = fg;
            return;
        }


        if(vyhodFG!=null){
            SachApp.gameStatus.getKontajner().vloz(vyhodFG);
        }

        vybrana = null;
        farba = getOponentColor();
        if(SachMatDetektor.detekujSachMat(bkr, sach)){
            fireMat(Color.biela);
        } else if(SachMatDetektor.detekujSach(bkr, sach)){
            fireSach(Color.biela);
        }
        if(SachMatDetektor.detekujSachMat(ckr, sach)){
            fireMat(Figurka.Color.cierna);
        } else if(SachMatDetektor.detekujSach(ckr, sach)){
            fireSach(Color.cierna);
        }
    }
    //</editor-fold>

    //<editor-fold defaultstate="collapsed" desc="GET/SET metody">
    /**
     * Vracia kópiu poľa s rozložením figúriek na hracej ploche
     * @return Kópia poľa s rozložením figúriek na hracej ploche
     */
    public FigurkaInterface[][] getSach(){
        return sach.clone();
    }

    /**
     * Vracia farbu figúriek hráča, ktorý je na ťahu
     * @return Farba figúriek hráča, ktorý je na ťahu
     */
    private Figurka.Color getPlayerColor(){
        return this.farba;
    }

    /**
     * Vracia farbu figúriek hráča, ktorý nie je na ťahu
     * @return Farba figúriek hráča, ktorý nie je na ťahu
     */
    private Figurka.Color getOponentColor(){
        return (getPlayerColor() == Figurka.Color.biela)? Color.cierna : Color.biela;
    }
    //</editor-fold>

    /**
     * Volaná na komponente pri ukončovaní programu, nemá bližší význam
     */
    public void exit() {
        try {
            finalize();
        } catch (Throwable ex) {
        }
    }
}
