package sach.logic;

/**
 * Implementácia figúrky typu strelec
 * @author Adam Netočný
 */
class Strelec extends Figurka implements FigurkaInterface {
    /**
     * Vytvorí novú figúrku typu strelec
     * @param x X-ová pozícia, na ktorej je strelec umiestnený
     * @param y Y-ová pozícia, na ktorej je strelec umiestnený
     * @param farba Farba novovytvorenej figúrky
     */
    Strelec(int x, int y, Figurka.Color farba){
        super(x, y, farba);
        typ = Figurka.Type.strelec;
    }

    /**
     * Kontrola ťahu figúrkou typu strelec
     * @param x X-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param y Y-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param sach Udáva pole, v ktorom sú rozmiestnené figúrky pričom sa berie v úvahu aj rozmiestnenie ostatných figúriek
     * @param detKral Určuje či má, alebo nemá brať v úvahu, že figúrkou, na ktorú sa snaží preisť je kráľ
     * @return true, ak sa môže figúrka presunúť, inak vráti false
     */
    @Override
    public boolean tah(int x, int y, FigurkaInterface[][] sach, boolean detKral) {
        if((this.x<8 && this.x>=0) && (this.y<8 && this.y>=0) && (x<8 && x>=0) && (y<8 && y>=0)){
            if(Math.abs(this.x-x)==Math.abs(this.y-y)){
                if(getFigrkaDirectionXY(sach, this.y<y, this.x<x)==null){           
                    return true;
                } else if(getFigrkaDirectionXY(sach, this.y<y, this.x<x).getX()==x
                        && getFigrkaDirectionXY(sach, this.y<y, this.x<x).getY()==y
                        && getFigrkaDirectionXY(sach, this.y<y, this.x<x).getColor()!=farba
                        && (getFigrkaDirectionXY(sach, this.y<y, this.x<x).getType()!=Figurka.Type.kral || !detKral)){
                    return true;
                } else if(((this.x<x) ?
                        getFigrkaDirectionXY(sach, this.y<y, this.x<x).getX()>x :
                        getFigrkaDirectionXY(sach, this.y<y, this.x<x).getX()<x)
                        && ((this.y<y)?
                        getFigrkaDirectionXY(sach, this.y<y, this.x<x).getY()>y :
                        getFigrkaDirectionXY(sach, this.y<y, this.x<x).getY()<y)){
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     * Vracia reťazcovú reprezentáciu danej figúrky
     * @return Reťazec vo formáte "Str" + "B" ako biely alebo "C" ako čierny
     */
    @Override
    public String toString() {
        return "Str" + ((farba==Figurka.Color.biela)? "B" : "C");
    }
}
