package sach.logic;

/**
 * Implementácia figúrky typu veža
 * @author Adam Netočný
 */
class Veza extends Figurka implements FigurkaInterface {
    /**
     * Uchováva logickú hodnotu o tom, či už veža urobila prvý ťah
     */
    private boolean prvyTah;

    /**
     * Vytvorí novú figúrku typu veža
     * @param x X-ová pozícia, na ktorej je veža umiestnená
     * @param y Y-ová pozícia, na ktorej je veža umiestnená
     * @param farba Farba novovytvorenej figúrky
     */
    Veza(int x, int y, Figurka.Color farba){
        super(x, y, farba);
        prvyTah = true;
        typ = Figurka.Type.veza;
    }

     /**
     * Kontrola ťahu figúrkou typu veža
     * @param x X-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param y Y-ová pozícia, na ktorú by sa mala figúrka presunúť
     * @param sach Udáva pole, v ktorom sú rozmiestnené figúrky pričom sa berie v úvahu aj rozmiestnenie ostatných figúriek
     * @param detKral Určuje či má, alebo nemá brať v úvahu, že figúrkou, na ktorú sa snaží preisť je kráľ
     * @return Vracia true, ak sa môže figúrka presunúť, inak vráti false
     */
    @Override
    public boolean tah(int x, int y, FigurkaInterface[][] sach, boolean detKral) {
        if((this.x<8 && this.x>=0) && (this.y<8 && this.y>=0) && (x<8 && x>=0) && (y<8 && y>=0)){
            if(this.x == x){
                if(getFigrkaDirectionY(sach, this.y<y)==null){
                    prvyTah = false;
                    return true;
                } else if(getFigrkaDirectionY(sach, this.y<y).getColor()!=farba
                        && getFigrkaDirectionY(sach, this.y<y).getX()==x
                        && getFigrkaDirectionY(sach, this.y<y).getY()==y
                        && (getFigrkaDirectionY(sach, this.y<y).getType()!=Figurka.Type.kral || !detKral)){
                    prvyTah = false;                    
                    return true;
                } else if((this.y<y)?
                        getFigrkaDirectionY(sach, true).getY()>y :
                        getFigrkaDirectionY(sach, false).getY()<y){

                    prvyTah = false;                   
                    return true;
                } else {
                    return false;
                }
            } else if(this.y == y) {
                if(getFigrkaDirectionX(sach, this.x<x)==null){
                    prvyTah = false;          
                    return true;
                } else if(getFigrkaDirectionX(sach, this.x<x).getColor()!=farba
                        && getFigrkaDirectionX(sach, this.x<x).getX()==x
                        && getFigrkaDirectionX(sach, this.x<x).getY()==y
                        && (getFigrkaDirectionX(sach, this.x<x).getType()!=Figurka.Type.kral || !detKral)){
                    prvyTah = false;
                    return true;
                } else if((this.x<x)?
                        getFigrkaDirectionX(sach, true).getX()>x :
                        getFigrkaDirectionX(sach, false).getX()<x){
                    prvyTah = false;      
                    return true;
                } else {
                    return false;
                }
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    /**
     * Detekuje, či figúrka urobila prvý ťah (potrebné pre detekciu rozšády)
     * @return true - neurobila, false - urobila
     */
    public boolean getPrvyTah(){
        return prvyTah;
    }

    /**
     * Vracia reťazcovú reprezentáciu danej figúrky
     * @return Reťazec vo formáte "Vez" + "B" ako biely alebo "C" ako čierny
     */
    @Override
    public String toString() {
        return "Vez" + ((farba==Figurka.Color.biela)? "B" : "C");
    } 
}