package sach.network;

import java.io.*;
import java.net.*;

/**
 * Sieťové rozhrannie vo forme klienta
 * @author Adam Netočný
 */
public class Client extends NetworkGame implements NetworkGameInterface {
    /**
     * Vytvorí klientské pripojenie pre sieťovú hru
     * @param ip IP adresa, na ktorú sa má pripojiť
     * @param port Číslo portu
     * @return true - úspešne pripojený, false - nepripojený
     */
    @Override
    public boolean connect(String ip, int port) throws NetworkGameException, NetworkTimeOutException, NetworkConnectionRefusedException {
        try {
            connection = new Socket(ip, port);
            connection.setKeepAlive(false);
            //pripojeny
        } catch (Exception ex) {
            //nepodarilo sa pripojit
            return false;
        }
        try {
            outStream = connection.getOutputStream();

            inStream = connection.getInputStream();
            //vytvoreny imput stream
        } catch (Exception ex) {
            closeConnection();
            return false;
        }
        if(!isConnected()){
            throw new NetworkGameException("Program nie je pripojeny!");
        }

        read();
            
        if(message.trim().equals("")){
            throw new NetworkGameException("Program nedostal pozadovane data!");
        }
        
        hostName = message;
        ipAddress = ip;

        sendResponse();

        waitForFirst();

        return true;
    }

    /**
     * Kontroluje, či je hra pripojená
     * @return true - pripojená, false - odpojená
     */
    @Override
    public boolean isConnected() {
        if(connection!=null){
            try {
                write("TEST");
                Thread.sleep(100);
                write("TEST");
            } catch (Throwable ex) { }
            return !connection.isClosed() && connection.isConnected()
                && !connection.isInputShutdown() && !connection.isOutputShutdown();
        } else {
            return false;
        }
    }

    /**
     * Ukončí pripojenie
     */
    @Override
    protected void closeConnection() throws NetworkGameException {
        try{
            if(isConnected()){
                try {
                    write("QUIT");
                } catch (Exception ex){}
                try {
                    connection.getInputStream().close();
                } catch (IOException ex) {}
                try {
                    connection.shutdownInput();
                } catch (IOException ex) {}
                try {
                    connection.getOutputStream().close();
                } catch (IOException ex) {}
                try {
                    connection.shutdownOutput();
                } catch (IOException ex) {}
                try {
                    connection.close();
                } catch (IOException ex) {}
                try {
                    inStream.close();
                } catch (IOException ex) {}
                try {
                    outStream.close();
                } catch (IOException ex) {}
            }           
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new NetworkGameException("Chyba pri odpojeni!");
        }
    }
}
