package sach.network;

import sach.ComponentInterface;
import sach.network.NetworkGame.NetworkConnectionRefusedException;

/**
 * Reprezentuje sieťové rozhrannie, či už vo forme servru alebo klienta
 * @author Adam Netočný
 */
public interface NetworkGameInterface extends ComponentInterface {

    public void exit();

    /**
     * Vytvorí pripojenie pre sieťovú hru
     * @param ip IP adresa, ne ktorú sa má pripojiť
     * @param port Číslo portu
     * @return true - úspešne pripojený, false - nepripojený
     * @throws NetworkGame.NetworkGameException 
     * @throws NetworkConnectionRefusedException
     * @throws NetworkGame.NetworkTimeOutException
     */
    boolean connect(String ip, int port) throws NetworkGame.NetworkGameException, NetworkConnectionRefusedException, NetworkGame.NetworkTimeOutException;
    /**
     * Vráti aktuálnu IP adresu serveru
     * @return IP adresa servru
     */
    String getIpAddress();
    /**
     * Vráti Host Name servru
     * @return Host Name servru
     */
    String getHostName();
    /**
     * Kontroluje, či je hra pripojená
     * @return true - pripojená, false - odpojená
     */
    boolean isConnected();
}
