package sach.network;

import java.io.IOException;
import java.net.*;

/**
 * Sieťové rozhrannie vo forme servru
 * @author Adam Netočný
 */
public class Server extends NetworkGame implements NetworkGameInterface {
    /**
     * Serverový soket
     */
    private ServerSocket listenSocket;

    /**
     * Vytvorí server a pripojenie pre sieťovú hru
     * @param ip Pri servri nemá význam, keďže si srver zaisťuje adresu sám
     * @param port Číslo portu
     * @return true - úspešne pripojený, false - nepripojený
     * @throws NetworkGame.NetworkGameException
     * @throws NetworkGame.NetworkTimeOutException
     */
    @Override
    public boolean connect(String ip, int port) throws NetworkGame.NetworkGameException, NetworkGame.NetworkTimeOutException, NetworkConnectionRefusedException {
        try {
            listenSocket = new ServerSocket(port);
            //server zacal

            InetAddress here = InetAddress.getLocalHost();
            hostName = here.getHostName();
            ipAddress = here.getHostAddress();
            //nazov hostu
        
            connection = listenSocket.accept();
            connection.setKeepAlive(false);
            
            outStream = connection.getOutputStream();

            inStream = connection.getInputStream();
            //vytvoreny imput stream
        } catch (Exception ex) {
            return false;
        }

        if(!isConnected()){
            throw new NetworkGameException("Program nie je pripojeny!");
        }

        write(hostName);

        if(!getResponse()){
            throw new NetworkGame.NetworkGameException("Program nedostal odozvu po odoslani dat!");
        } else {
            return true;
        }
    }

    /**
     * Kontroluje, či je hra pripojená
     * @return true - pripojená, false - odpojená
     */
    @Override
    public boolean isConnected() {
        return isServerConnected() && isConnectionConnected();
    }

    /**
     * Kontroluje, či existuje servrový soket a či je aktívny
     * @return true - aktívny, false - odpojený
     */
    private boolean isServerConnected(){
        if(listenSocket!=null){
            try {
                write("TEST");
                Thread.sleep(100);
                write("TEST");
            } catch (Throwable ex) { }
            return listenSocket.isBound() && !listenSocket.isClosed();
        } else {
            return false;
        }
    }

    /**
     * Kontroluje, či je hra prepojená
     * @return true - prepojená, false - odpojená
     */
    private boolean isConnectionConnected(){
        if(connection!=null){
            try {
                write("TEST");
                Thread.sleep(100);
                write("TEST");
            } catch (Throwable ex) { }
            return !connection.isClosed() && connection.isConnected()
                && !connection.isInputShutdown() && !connection.isOutputShutdown();
        } else {
            return false;
        }
    }

    /**
     * Ukončí pripojenie
     */
    @Override
    protected void closeConnection() throws NetworkGameException {
        try{
            if(isConnected()) {
                try{ 
                    write("QUIT");
                } catch (Throwable ex){}
            }
            //zrusi server
            if(isServerConnected()){
                try {
                    listenSocket.close();
                } catch (IOException ex) {}
            }
            //zrusi premostenie
            if(isConnectionConnected()){
                try {
                    connection.getInputStream().close();
                } catch (IOException ex) {}
                try {
                    connection.shutdownInput();
                } catch (IOException ex) {}
                try {
                    connection.getOutputStream().close();
                } catch (IOException ex) {}
                try {
                    connection.shutdownOutput();
                } catch (IOException ex) {}
                try {
                    connection.close();
                } catch (IOException ex) {}
                try {
                    inStream.close();
                } catch (IOException ex) {}
                try {
                    outStream.close();
                } catch (IOException ex) {}
            }    
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new NetworkGameException("Chyba pri odpojeni!");
        }
    }
}
