package sach.logic;

import sach.logic.Dama;
import sach.logic.Pesiak;
import sach.logic.FigurkaInterface;
import org.junit.Test;
import static org.junit.Assert.*;
import sach.logic.Figurka.PesiakNaKonciPola;
import sach.logic.Figurka.Rozsada;

public class DamaUnitTest {

    @Test
    public void tah() throws PesiakNaKonciPola, Rozsada{
        /*| |0 |1 |2 |3 |4 |5 |6 |7 |
          --------------------------|
          |0|PB|  |  |X |  |  |PB|  |
          |1|  |X |  |PB|  |X |  |  |
          |2|  |X |  |X |  |  |  |  |
          |3|X |  |  |DB|X |PC|X |  |
          |4|  |  |  |X |  |  |X |  |
          |5|  |PC|  |PC|  |PB|  |  |
          |6|X |  |  |X |  |  |X |  |
          |7|  |  |  |  |  |  |  |  |
         V-veza
         K-kon
         S-strelec
         D-dama
         P-pesiak
         KR-kral

         B-biela farba
         C-cierna farba
         */
        FigurkaInterface[][] pole = new FigurkaInterface[8][8];

        Dama dama = new Dama(3, 3, Figurka.Color.biela);
        pole[3][3] = dama;

        pole[0][0] = new Pesiak(0, 0, Figurka.Color.biela);
        pole[0][6] = new Pesiak(0, 6, Figurka.Color.biela);
        pole[1][3] = new Pesiak(1, 3, Figurka.Color.biela);
        pole[5][5] = new Pesiak(5, 5, Figurka.Color.biela);

        pole[5][1] = new Pesiak(5, 1, Figurka.Color.cierna);
        pole[5][3] = new Pesiak(5, 3, Figurka.Color.cierna);
        pole[3][5] = new Pesiak(3, 5, Figurka.Color.cierna);

        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(1, 1, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(0, 0, pole), false);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(-2, 8, pole), false);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(5, 5, pole), false);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(1, 5, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(5, 1, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(6, 0, pole), false);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(0, 6, pole), false);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(3, 5, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(4, 6, pole), false);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(6, 6, pole), false);
        pole[5][5] = null;
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(6, 6, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(3, 0, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(2, 1, pole), false);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(-2, -8, pole), false);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(0, 3, pole), false);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(6, 3, pole), false);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        pole[5][3] = null;
        assertEquals(dama.tah(6, 3, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        pole[5][3] = new Pesiak(5, 3, Figurka.Color.cierna);
        assertEquals(dama.tah(5, 3, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(3, 5, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(4, 3, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(3, 4, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(2, 3, pole), true);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        assertEquals(dama.tah(3, 6, pole), false);
        assertEquals(dama.getX(), 3);
        assertEquals(dama.getY(), 3);
        dama.setXY(2,0);
        assertEquals(dama.getX(), 2);
        assertEquals(dama.getY(), 0);
    }
}
