package sach.logic;

import sach.logic.Pesiak;
import sach.logic.FigurkaInterface;
import sach.logic.Kral;
import sach.logic.Veza;
import sach.logic.Strelec;
import org.junit.Test;
import static org.junit.Assert.*;
import sach.logic.Figurka.PesiakNaKonciPola;
import sach.logic.Figurka.Rozsada;

public class KralUnitTest {

    @Test
    public void tah() throws PesiakNaKonciPola, Rozsada{
        /*| |0 |1 |2 |3  |4  |5 |6 |7 |
          ----------------------------|
          |0|  |  |  |   |   |  |  |  |
          |1|  |  |  |   |   |  |  |  |
          |2|  |  |X |PB |X  |  |  |  |
          |3|  |  |X |KRB|PC |  |  |  |
          |4|  |X |X |X  |X  |  |  |  |
          |5|  |  |  |KRC|   |  |  |  |
          |6|  |X |  |   |   |  |CK|  |
          |7|  |  |  |   |KRB|SB|  |VB|
         V-veza
         K-kon
         S-strelec
         D-dama
         P-pesiak
         KR-kral

         B-biela farba
         C-cierna farba
         */
        FigurkaInterface[][] pole = new FigurkaInterface[8][8];

        Kral kral = new Kral(3, 3, Figurka.Color.biela);
        pole[3][3] = kral;
        pole[5][3] = new Kral(5,3, Figurka.Color.cierna);
        pole[2][3] = new Pesiak(2, 3, Figurka.Color.biela);

        pole[3][4] = new Pesiak(3, 4, Figurka.Color.cierna);

        Veza veza = new Veza(7, 7, Figurka.Color.biela);
        pole[7][7] = veza;
        pole[7][0] = new Veza(7, 0, Figurka.Color.cierna);

        assertEquals(kral.getX(), 3);
        assertEquals(kral.getY(), 3);
        assertEquals(kral.tah(-2, 2, pole), false);
        assertEquals(kral.getX(), 3);
        assertEquals(kral.getY(), 3);
        assertEquals(kral.tah(2, 2, pole), true);
        assertEquals(kral.getX(), 3);
        assertEquals(kral.getY(), 3);
        assertEquals(kral.tah(2, 3, pole), false);
        assertEquals(kral.getX(), 3);
        assertEquals(kral.getY(), 3);
        assertEquals(kral.tah(2, 4, pole), true);
        assertEquals(kral.getX(), 3);
        assertEquals(kral.getY(), 3);
        assertEquals(kral.tah(3, 2, pole), true);
        assertEquals(kral.getX(), 3);
        assertEquals(kral.getY(), 3);
        assertEquals(kral.tah(3, 4, pole), true);
        assertEquals(kral.getX(), 3);
        assertEquals(kral.getY(), 3);
        assertEquals(kral.tah(4, 2, pole), false);
        assertEquals(kral.getX(), 3);
        assertEquals(kral.getY(), 3);
        assertEquals(kral.tah(4, 3, pole), false);
        assertEquals(kral.getX(), 3);
        assertEquals(kral.getY(), 3);
        assertEquals(kral.tah(4, 4, pole), false);
        assertEquals(kral.getX(), 3);
        assertEquals(kral.getY(), 3);
        assertEquals(kral.tah(4, 1, pole), false);
        assertEquals(kral.getX(), 3);
        assertEquals(kral.getY(), 3);
        assertEquals(kral.tah(6, 1, pole), false);

        kral.setXY(7,4);
        assertEquals(kral.getX(), 7);
        assertEquals(kral.getY(), 4);
        kral = new Kral(7, 4, Figurka.Color.biela);
        pole[7][4] = kral;
        try {
            kral.tah(7, 7, pole);
            assertTrue(false);
        } catch (Rozsada ex){
            assertTrue(true);
        }

        assertEquals(kral.getX(), 7);
        assertEquals(kral.getY(), 4);
        kral = new Kral(7, 4, Figurka.Color.biela);
        kral.setPrvyTah(false);
        pole[7][4] = kral;
        try {
            kral.tah(7, 7, pole);
            assertTrue(true);
        } catch (Rozsada ex){
            assertTrue(false);
        }

        assertEquals(kral.getX(), 7);
        assertEquals(kral.getY(), 4);
        kral = new Kral(7, 4, Figurka.Color.biela);
        kral.setBolVSachu(true);
        pole[7][4] = kral;
        try {
            kral.tah(7, 7, pole);
            assertTrue(true);
        } catch (Rozsada ex){
            assertTrue(false);
        }
        
        kral = new Kral(7, 4, Figurka.Color.biela);
        pole[7][4] = kral;
        pole[7][5] = new Strelec(7, 5, Figurka.Color.biela);
        try {
            kral.tah(7, 7, pole);
            assertTrue(true);
        } catch (Rozsada ex){
            assertTrue(false);
        }

        kral = new Kral(7, 4, Figurka.Color.biela);
        pole[7][4] = kral;
        pole[7][5] = null;
        pole[6][6] = new Kral(6, 6, Figurka.Color.cierna);
        try {
            kral.tah(7, 7, pole);
            assertTrue(true);
        } catch (Rozsada ex){
            assertTrue(false);
        }

        veza.tah(7, 6, pole);
        kral = new Kral(7, 4, Figurka.Color.biela);
        pole[7][4] = kral;
        pole[6][6] = null;
        try {
            kral.tah(7, 7, pole);
            assertTrue(true);
        } catch (Rozsada ex){
            assertTrue(false);
        }
        kral = new Kral(7, 4, Figurka.Color.biela);
        pole[7][4] = kral;
        try {
            kral.tah(7, 0, pole);
            assertTrue(true);
        } catch (Rozsada ex){
            assertTrue(false);
        }

        assertEquals(kral.toString(), "KrlB");
    }
}
