package sach.logic;

import org.junit.Test;
import static org.junit.Assert.*;
import sach.logic.Figurka.PesiakNaKonciPola;
import sach.logic.Figurka.Rozsada;

public class PesiakUnitTest {

    @Test
    public void tah() throws PesiakNaKonciPola, Rozsada{
        /*| |0 |1 |2 |3 |4 |5 |6 |7 |
          --------------------------|
          |0|  |  |  |  |PB|X |PC|  |
          |1|  |  |  |  |  |PB|  |  |
          |2|  |  |  |X |  |  |X |  |
          |3|  |  |PC|PC|X |  |  |  |
          |4|  |  |PB|PB|  |  |  |  |
          |5|  |  |  |X |  |X |  |  |
          |6|  |  |  |  |  |  |  |  |
          |7|  |  |  |  |  |  |  |  |
         V-veza
         K-kon
         S-strelec
         D-dama
         P-pesiak
         KR-kral

         B-biela farba
         C-cierna farba
         */
        FigurkaInterface[][] pole = new FigurkaInterface[8][8];

        Pesiak pesiak = new Pesiak(4, 3, Figurka.Color.biela);
        pole[4][3] = pesiak;

        pole[0][4] = new Pesiak(0, 4, Figurka.Color.biela);
        pole[3][2] = new Pesiak(3, 2, Figurka.Color.cierna);
        pole[0][6] = new Pesiak(0, 6, Figurka.Color.cierna);
        pole[3][3] = new Pesiak(3, 3, Figurka.Color.cierna);

        assertEquals(pesiak.tah(5, 3, pole), false);
        assertEquals(pesiak.tah(2, 3, pole), false);
        assertEquals(pesiak.tah(3, 3, pole), false);

        pole[3][3] = null;
        assertEquals(pesiak.tah(2, 3, pole), true);
        assertEquals(pesiak.tah(3, 3, pole), true);

        assertEquals(pesiak.tah(3, 2, pole), true);
        assertEquals(pesiak.tah(3, 4, pole), false);

        assertEquals(pesiak.tah(-3, 4, pole), false);
        assertEquals(pesiak.tah(5, 5, pole), false);
        assertEquals(pesiak.tah(2, 6, pole), false);


        pole[4][2] = new Pesiak(4, 2,Figurka.Color.biela);
        assertEquals(pole[3][2].tah(5, 2, pole), false);
        assertEquals(pole[3][2].tah(4, 2, pole), false);
        pole[4][2] = null;
        assertEquals(pole[3][2].tah(5, 2, pole), true);
        assertEquals(pole[3][2].tah(4, 2, pole), true);
        
        assertEquals(pole[3][2].tah(4, 3, pole), true);
        assertEquals(pole[3][2].tah(2, 2, pole), false);
        

        pesiak.setXY(1,5);
        assertEquals(pesiak.getX(), 1);
        assertEquals(pesiak.getY(), 5);
        pole[4][3] = null;
        pole[1][5] = pesiak;
        
        assertEquals(pesiak.tah(0, 4, pole), false);

        try {
            pesiak.tah(0, 5, pole);
            assertTrue(false);
        } catch(PesiakNaKonciPola ex){
            assertTrue(true);
        }

        try {
            pesiak.tah(0, 6, pole);
            assertTrue(false);
        } catch(PesiakNaKonciPola ex){
            assertTrue(true);
        }
    }
}
