package sach.logic;

import org.junit.Test;
import static org.junit.Assert.*;
import sach.network.NetworkGame.*;

public class SachMatUnitTest {
    @Test
    public void sach() {
        /*| |0 |1 |2 |3  |4 |5 |6 |7 |
          ---------------------------|
          |0|  |  |  |   |  |  |  |  |
          |1|  |  |  |   |  |  |DC|  |
          |2|  |  |  |   |  |  |  |  |
          |3|  |  |PC|   |  |  |  |  |
          |4|  |  |  |KRB|  |  |DC|  |
          |5|  |  |  |   |  |  |  |  |
          |6|  |  |KC|   |  |  |  |  |
          |7|  |  |  |VC |  |  |SC|  |
         V-veza
         K-kon
         S-strelec
         D-dama
         P-pesiak
         KR-kral

         B-biela farba
         C-cierna farba
         */

        FigurkaInterface[][] pole = new FigurkaInterface[8][8];

        Kral kr = new Kral(4, 3, Figurka.Color.biela);
        pole[4][3] = kr;

        pole[3][2] = new Pesiak(3, 2, Figurka.Color.cierna);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));
        pole[3][2] = null;

        pole[6][2] = new Kon(6, 2, Figurka.Color.cierna);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));
        pole[6][2] = null;

        pole[7][3] = new Veza(7, 3, Figurka.Color.cierna);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));
        pole[7][3] = null;

        pole[7][6] = new Strelec(7, 6, Figurka.Color.cierna);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));
        pole[7][6] = null;

        pole[4][6] = new Dama(4, 6, Figurka.Color.cierna);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));
        pole[4][6] = null;

        pole[1][6] = new Dama(1, 6, Figurka.Color.cierna);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));

        kr = new Kral(4, 3, Figurka.Color.cierna);
        pole[4][3] = kr;

        pole[3][2] = new Pesiak(3, 2, Figurka.Color.biela);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));
        pole[3][2] = null;

        pole[6][2] = new Kon(6, 2, Figurka.Color.biela);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));
        pole[6][2] = null;

        pole[7][3] = new Veza(7, 3, Figurka.Color.biela);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));
        pole[7][3] = null;

        pole[7][6] = new Strelec(7, 6, Figurka.Color.biela);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));
        pole[7][6] = null;

        pole[4][6] = new Dama(4, 6, Figurka.Color.biela);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));
        pole[4][6] = null;

        pole[1][6] = new Dama(1, 6, Figurka.Color.biela);
        assertTrue(SachMatDetektor.detekujSach(kr, pole));
    }

    @Test
    public void mat() {
        /*| |0  |1 |2 |3 |4 |5  |6 |7 |
          ----------------------------|
          |0|   |  |DB|  |KC|VB |  |  |
          |1|   |  |  |KB|PC|   |  |  |
          |2|   |  |  |  |  |SC |PC|  |
          |3|   |  |  |  |  |   |PC|  |
          |4|   |  |  |PB|SB|KRC|PC|  |
          |5|   |  |PC|  |PC|   |KB|SC|
          |6|   |  |PB|  |  |   |  |PB|
          |7|KRB|  |  |  |VB|   |  |  |
         V-veza
         K-kon
         S-strelec
         D-dama
         P-pesiak
         KR-kral

         B-biela farba
         C-cierna farba
         */

        FigurkaInterface[][] pole = new FigurkaInterface[8][8];
        FigurkaInterface fg = null;

        Kral krc = new Kral(4, 5, Figurka.Color.cierna);
        pole[4][5] = krc;

        pole[0][2] = new Dama(0, 2, Figurka.Color.biela);
        pole[0][4] = new Kon(0, 4, Figurka.Color.cierna);
        pole[0][5] = new Veza(0, 5, Figurka.Color.biela);

        pole[1][3] = new Kon(1, 3, Figurka.Color.biela);
        pole[1][4] = new Pesiak(1, 4, Figurka.Color.cierna);

        pole[2][5] = new Strelec(2, 5, Figurka.Color.cierna);
        pole[2][6] = new Pesiak(2, 6, Figurka.Color.cierna);

        pole[3][6] = new Pesiak(3, 6, Figurka.Color.cierna);

        pole[4][3] = new Pesiak(4, 3, Figurka.Color.biela);
        pole[4][4] = new Strelec(4, 4, Figurka.Color.biela);
        pole[4][6] = new Pesiak(0, 6, Figurka.Color.cierna);
        pole[4][6].setXY(4, 6);

        pole[5][2] = new Pesiak(0, 2, Figurka.Color.cierna);
        pole[5][2].setXY(5, 2);
        pole[5][4] = new Pesiak(0, 4, Figurka.Color.cierna);
        pole[5][4].setXY(5, 4);
        pole[5][6] = new Kon(5, 6, Figurka.Color.biela);
        pole[5][7] = new Strelec(5, 7, Figurka.Color.cierna);

        pole[6][2] = new Pesiak(6, 2, Figurka.Color.biela);
        pole[6][7] = new Pesiak(6, 7, Figurka.Color.biela);

        Kral krb = new Kral(7, 0, Figurka.Color.biela);
        pole[7][0] = krb;
        pole[7][4] = new Veza(7, 4, Figurka.Color.biela);

        System.out.println("Sach: false; Mat: true");
        write(pole);
        
        assertFalse(SachMatDetektor.detekujSach(krc, pole));

        assertTrue(SachMatDetektor.detekujMat(krc, pole));

        fg = pole[7][4];
        pole[7][4] = null;
        pole[7][5] = fg;
        pole[7][5].setXY(7, 5);

        System.out.println("Sach: true; Mat: false");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertFalse(SachMatDetektor.detekujMat(krc, pole));

        fg = pole[7][5];
        pole[7][5] = null;
        pole[5][5] = fg;
        pole[5][5].setXY(5, 5);

        System.out.println("Sach: true; Mat: false");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertFalse(SachMatDetektor.detekujMat(krc, pole));

        fg = pole[4][6];
        pole[4][6] = null;

        System.out.println("Sach: true; Mat: false");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertFalse(SachMatDetektor.detekujMat(krc, pole));
        
        pole[5][7] = new Strelec(5, 7, Figurka.Color.biela);

        System.out.println("Sach: true; Mat: true");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertTrue(SachMatDetektor.detekujMat(krc, pole));

        fg = pole[5][5];
        pole[5][5] = null;
        pole[5][4] = null;
        pole[6][5] = fg;
        fg.setXY(6, 5);
        pole[7][4] = new Veza(7, 4, Figurka.Color.biela);

        System.out.println("Sach: true; Mat: true");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertTrue(SachMatDetektor.detekujMat(krc, pole));

        pole[5][4] = new Veza(5, 4, Figurka.Color.cierna);

        System.out.println("Sach: true; Mat: false");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertFalse(SachMatDetektor.detekujMat(krc, pole));

        pole[6][5] = new Dama(6, 5, Figurka.Color.biela);

        System.out.println("Sach: true; Mat: false");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertFalse(SachMatDetektor.detekujMat(krc, pole));

        pole[5][4] = null;

        System.out.println("Sach: true; Mat: true");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertTrue(SachMatDetektor.detekujMat(krc, pole));

        pole[6][5] = null;

        System.out.println("Sach: false; Mat: true");
        write(pole);

        assertFalse(SachMatDetektor.detekujSach(krc, pole));

        assertTrue(SachMatDetektor.detekujMat(krc, pole));

        fg = pole[0][2];
        pole[0][2] = null;
        pole[0][1] = fg;
        fg.setXY(0, 1);

        System.out.println("Sach: true; Mat: false");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertFalse(SachMatDetektor.detekujMat(krc, pole));

        pole[0][4] = null;

        System.out.println("Sach: true; Mat: false");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertFalse(SachMatDetektor.detekujMat(krc, pole));

        pole[2][5] = null;
        pole[0][5] = null;

        System.out.println("Sach: true; Mat: false");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertFalse(SachMatDetektor.detekujMat(krc, pole));

        pole[1][4] = null;

        System.out.println("Sach: true; Mat: true");
        write(pole);

        assertTrue(SachMatDetektor.detekujSach(krc, pole));

        assertTrue(SachMatDetektor.detekujMat(krc, pole));
    }

    private void write(FigurkaInterface[][] pole){
        for(int a = -1; a < 8; a++){
            for(int  b = 0; b < 8; b++){
                if(a==-1){
                    System.out.print("|" + (b+1) + "   ");
                } else {
                    if(pole[a][b]!=null){
                        System.out.print("|" +pole[a][b].toString());
                    } else {
                        System.out.print("|" + "    ");
                    }
                }
            }
            System.out.println("|" + (a+1));
            System.out.println("-----------------------------------------");
        }
        System.out.println("");
        System.out.println("");
    }
}
