package sach.logic;

import sach.*;
import sach.eventmodel.TahEvent;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.junit.Test;
import sach.eventmodel.TahHandler;
import static org.junit.Assert.*;

public class SachovnicaUnitTest {
    private GameStatus st;

    private class Handler extends TahHandler {
        private int potvrdenie = 0;
        @Override
        public void tah(TahEvent event) {
           potvrdenie = 0;
           assertTrue(true);
        }

        @Override
        public void chybyTah(TahEvent event) {
           potvrdenie = 2;
           assertTrue(false);
        }

        @Override
        public void vyberFigurku(TahEvent event) {
            if(!event.equals(this.event)){
                if(event.getTypFig() == null){
                    fireVyberFigurku(event.getOdkial(), event.getKam(), Figurka.Type.dama, event.getFarba());
                } else {
                    fireVyberFigurku(event.getOdkial(), event.getKam(), event.getTypFig(), event.getFarba());
                }
            } else {
                potvrdenie = 0;
            }
        }

        @Override
        public void chyba(TahEvent event) {
           if(!event.equals(this.event)){
                fireChyba(event.getChyba());
            }
            System.out.println(event.getChyba());
        }

        @Override
        public void sach(TahEvent event) {
            if(!event.equals(this.event)){
                fireSach(event.getFarba());
            }
            System.out.println(((event.getFarba() == Figurka.Color.biela)? "Biely" : "Cierny")
                    + " kral dostal sach!");
        }

        @Override
        public void mat(TahEvent event) {
            if(!event.equals(this.event)){
                fireMat(event.getFarba());
            }
            System.out.println(((event.getFarba() == Figurka.Color.biela)? "Biely" : "Cierny")
                    + " kral dostal mat!");
        }

        public void tah(Point a, Point b) throws InterruptedException{
            while (potvrdenie != 0){
                if(potvrdenie == 2)
                    return;
                Thread.sleep(100);
            }
            potvrdenie = 1;
            fireTah(a, b, Figurka.Color.biela);
        }
    };

    @Test//Test podla realnej hry :)
    public void tah(){
        SachApp.test = true;
        Sachovnica sach = new Sachovnica();
        GameStatus status = new GameStatus(Figurka.Color.biela, sach);
        assertFalse(status.getGameStarted());
        assertEquals(status.getKontajner().getBieleLength(), 0);
        assertEquals(status.getKontajner().getCierneLength(), 0);
        assertEquals(status.getPlayerColor(), Figurka.Color.biela);
        status.setPlayerColor(Figurka.Color.cierna);
        assertEquals(status.getPlayerColor(), Figurka.Color.cierna);
        status.setGameSterted(true);
        assertTrue(status.getGameStarted());
        status.setPlayerColor(Figurka.Color.biela);
        assertEquals(status.getPlayerColor(), Figurka.Color.biela);
        
        st = new GameStatus(Figurka.Color.biela, sach);
        SachApp.gameStatus = st;

        Handler th = new Handler();

        th.addTahListener(sach);
        sach.addTahListener(th);

        FigurkaInterface[][] sch = sach.getSach();
        Kral krb = (Kral) sach.getSach()[7][4];
        Kral krc = (Kral) sach.getSach()[0][4];

        //D2 -> D4; Farba: biela
        move("D2", "D4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }
        
        //D7 -> D5; Farba: cierna
        move("D7", "D5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C2 -> C3; Farba: biela
        move("C2", "C3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //B7 -> B6; Farba: cierna
        move("B7", "B6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G2 -> G3; Farba: biela
        move("G2", "G3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C7 -> C5; Farba: cierna
        move("C7", "C5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //F1 -> G2; Farba: biela
        move("F1", "G2", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C8 -> B7; Farba: cierna
        move("C8", "B7", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G1 -> F3; Farba: biela
        move("G1", "F3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G7 -> G6; Farba: cierna
        move("G7", "G6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E1 -> H1; Farba: biela
        move("E1", "H1", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //F8 -> H6; Farba: cierna
        move("F8", "H6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E2 -> E3; Farba: biela
        move("E2", "E3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //B8 -> C6; Farba: cierna
        move("B8", "C6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //B1 -> D2; Farba: biela
        move("B1", "D2", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //B7 -> A6; Farba: cierna
        move("B7", "A6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //F1 -> E1; Farba: biela
        move("F1", "E1", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E7 -> E6; Farba: cierna
        move("E7", "E6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //D1 -> A4; Farba: biela
        move("D1", "A4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //D8 -> C8; Farba: cierna
        move("D8", "C8", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //D4 -> C5; Farba: biela
        move("D4", "C5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //B6 -> B5; Farba: cierna
        move("B6", "B5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //A4 -> H4; Farba: biela
        move("A4", "H4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //B5 -> B4; Farba: cierna
        move("B5", "B4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E3 -> E4; Farba: biela
        move("E3", "E4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H6 -> D2; Farba: cierna
        move("H6", "D2", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C1 -> D2; Farba: biela
        move("C1", "D2", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //B4 -> C3; Farba: cierna
        move("B4", "C3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //D2 -> C3; Farba: biela
        move("D2", "C3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //D5 -> D4; Farba: cierna
        move("D5", "D4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //F3 -> D4; Farba: biela
        move("F3", "D4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C6 -> D4; Farba: cierna
        move("C6", "D4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C3 -> D4; Farba: biela
        move("C3", "D4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C8 -> D8; Farba: cierna
        move("C8", "D8", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //D4 -> H8; Farba: biela
        move("D4", "H8", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //D8 -> H4; Farba: cierna
        move("D8", "H4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G3 -> H4; Farba: biela
        move("G3", "H4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //F7 -> F5; Farba: cierna
        move("F7", "F5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E4 -> F5; Farba: biela
        move("E4", "F5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E8 -> F7; Farba: cierna
        move("E8", "F7", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G2 -> A8; Farba: biela
        move("G2", "A8", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G8 -> H6; Farba: cierna
        move("G8", "H6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E1 -> E6; Farba: biela
        move("E1", "E6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H6 -> F5; Farba: cierna
        move("H6", "F5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E6 -> A6; Farba: biela
        move("E6", "A6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //F5 -> H4; Farba: cierna
        move("F5", "H4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //A6 -> A7; Farba: biela
        move("A6", "A7", sach, th);

        sch = sach.getSach();
        assertTrue(SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //F7 -> G8; Farba: cierna
        move("F7", "G8", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H8 -> D4; Farba: biela
        move("H8", "D4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H4 -> F3; Farba: cierna
        move("H4", "F3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G1 -> G2; Farba: biela
        move("G1", "G2", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //F3 -> D4; Farba: cierna
        move("F3", "D4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //A1 -> E1; Farba: biela
        move("A1", "E1", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //D4 -> F5; Farba: cierna
        move("D4", "F5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C5 -> C6; Farba: biela
        move("C5", "C6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //F5 -> D6; Farba: cierna
        move("F5", "D6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C6 -> C7; Farba: biela
        move("C6", "C7", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //D6 -> C8; Farba: cierna
        move("D6", "C8", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //A7 -> B7; Farba: biela
        move("A7", "B7", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G8 -> G7; Farba: cierna
        move("G8", "G7", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //B7 -> B8; Farba: biela
        move("B7", "B8", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C8 -> D6; Farba: cierna
        move("C8", "D6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C7 -> C8; Farba: biela
        move("C7", "C8", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //D6 -> C8; Farba: cierna
        move("D6", "C8", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //B8 -> C8; Farba: biela
        move("B8", "C8", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G7 -> H6; Farba: cierna
        move("G7", "H6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G2 -> G3; Farba: biela
        move("G2", "G3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H6 -> G5; Farba: cierna
        move("H6", "G5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E1 -> E4; Farba: biela
        move("E1", "E4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H7 -> H5; Farba: cierna
        move("H7", "H5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H2 -> H4; Farba: biela
        move("H2", "H4", sach, th);

        sch = sach.getSach();
        assertTrue(SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G5 -> H6; Farba: cierna
        move("G5", "H6", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E4 -> E7; Farba: biela
        move("E4", "E7", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G6 -> G5; Farba: cierna
        move("G6", "G5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H4 -> G5; Farba: biela
        move("H4", "G5", sach, th);

        sch = sach.getSach();
        assertTrue(SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H6 -> G5; Farba: cierna
        move("H6", "G5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G3 -> H3; Farba: biela
        move("G3", "H3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H5 -> H4; Farba: cierna
        move("H5", "H4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H3 -> G2; Farba: biela
        move("H3", "G2", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G5 -> G4; Farba: cierna
        move("G5", "G4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E7 -> H7; Farba: biela
        move("E7", "H7", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H4 -> H3; Farba: cierna
        move("H4", "H3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H7 -> H3; Farba: biela
        move("H7", "H3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G4 -> F5; Farba: cierna
        move("G4", "F5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //C8 -> E8; Farba: biela
        move("C8", "E8", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //F5 -> G4; Farba: cierna
        move("F5", "G4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //H3 -> F3; Farba: biela
        move("H3", "F3", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G4 -> G5; Farba: cierna
        move("G4", "G5", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //E8 -> G8; Farba: biela
        move("E8", "G8", sach, th);

        sch = sach.getSach();
        assertTrue(SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }

        //G5 -> H4; Farba: cierna
        move("G5", "H4", sach, th);

        sch = sach.getSach();
        assertTrue(!SachMatDetektor.detekujSach(krc, sch));
        if(SachMatDetektor.detekujSach(krc, sch)) { assertTrue(!SachMatDetektor.detekujMat(krc, sch)); }
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }
        
        //F3 -> H3; Farba: biela
        move("F3", "H3", sach, th);

        sch = sach.getSach();
        assertTrue(SachMatDetektor.detekujSach(krc, sch));
        assertTrue(SachMatDetektor.detekujMat(krc, sch));
        assertTrue(!SachMatDetektor.detekujSach(krb, sch));
        if(SachMatDetektor.detekujSach(krb, sch)) { assertTrue(!SachMatDetektor.detekujMat(krb, sch)); }
    }

    private void move(String z, String kam, Sachovnica sach, Handler th){
        try {
            System.out.println(z + " -> " + kam + ": " + st.getPlayerColor());
            st.setPlayerColor(st.getOponentColor());
            th.tah(new Point(z), new Point(kam));
        } catch (InterruptedException ex) {
            Logger.getLogger(SachovnicaUnitTest.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
