package sach.logic;

import sach.logic.Pesiak;
import sach.logic.FigurkaInterface;
import sach.logic.Veza;
import org.junit.Test;
import static org.junit.Assert.*;
import sach.logic.Figurka.PesiakNaKonciPola;
import sach.logic.Figurka.Rozsada;

public class VezaUnitTest {

    @Test
    public void tah() throws PesiakNaKonciPola, Rozsada{
        /*| |0 |1 |2 |3 |4 |5 |6 |7 |
          --------------------------|
          |0|  |  |  |X |  |  |  |  |
          |1|  |  |  |PB|  |  |  |  |
          |2|  |X |  |X |  |  |  |  |
          |3|X |  |  |VB|X |PC|X |  |
          |4|  |  |  |X |  |  |  |  |
          |5|  |  |  |PC|  |X |  |  |
          |6|  |  |  |X |  |  |  |  |
          |7|  |  |  |  |  |  |  |  |
         V-veza
         K-kon
         S-strelec
         D-dama
         P-pesiak
         KR-kral

         B-biela farba
         C-cierna farba
         */
        FigurkaInterface[][] pole = new FigurkaInterface[8][8];

        Veza veza = new Veza(3, 3, Figurka.Color.biela);
        pole[3][3] = veza;
        
        pole[1][3] = new Pesiak(1, 3, Figurka.Color.biela);

        pole[5][3] = new Pesiak(5, 3, Figurka.Color.cierna);
        pole[3][5] = new Pesiak(3, 5, Figurka.Color.cierna);

        
        assertEquals(veza.getPrvyTah(), true);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        assertEquals(veza.tah(3, 0, pole), true);

        assertEquals(veza.getPrvyTah(), false);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        assertEquals(veza.tah(2, 1, pole), false);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        assertEquals(veza.tah(-2, -8, pole), false);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        assertEquals(veza.tah(0, 3, pole), false);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        assertEquals(veza.tah(6, 3, pole), false);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        pole[5][3] = null;
        assertEquals(veza.tah(6, 3, pole), true);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        pole[5][3] = new Pesiak(5, 3, Figurka.Color.cierna);
        assertEquals(veza.tah(5, 3, pole), true);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        assertEquals(veza.tah(3, 5, pole), true);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        assertEquals(veza.tah(4, 3, pole), true);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        assertEquals(veza.tah(3, 4, pole), true);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        assertEquals(veza.tah(2, 3, pole), true);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        assertEquals(veza.tah(3, 6, pole), false);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        assertEquals(veza.tah(5, 5, pole), false);
        assertEquals(veza.getX(), 3);
        assertEquals(veza.getY(), 3);
        veza.setXY(2,0);
        assertEquals(veza.getX(), 2);
        assertEquals(veza.getY(), 0);
    }

    @Test
    public void vlastnosti(){
        Veza veza = new Veza(3, 3, Figurka.Color.biela);
        assertEquals(veza.getPrvyTah(), true);
        assertEquals(veza.getType(), Figurka.Type.veza);
        assertEquals(veza.getColor(), Figurka.Color.biela);
    }
}
