package sach.network;

import java.util.logging.Level;
import java.util.logging.Logger;
import sach.logic.Figurka;
import java.net.InetAddress;
import java.net.UnknownHostException;
import org.junit.Test;
import sach.SachApp;
import sach.eventmodel.*;
import sach.logic.Figurka.Type;
import sach.logic.GameStatus;
import sach.logic.Point;
import static org.junit.Assert.*;
import sach.network.NetworkGame.*;

public class NetworkGameUnitTest {
    final Server srv = new Server();
    Client clt = new Client();

    private class Handler extends TahHandler {
        private int odk;
        private int kam;
        private Figurka.Type typfig;
        private String nazov;
        public int prijal = 0;

        public Handler(int odk, int kam, Type typfig, String nazov) {
            this.odk = odk;
            this.kam = kam;
            this.typfig = typfig;
            this.nazov = nazov;
        }

        @Override
        public void tah(TahEvent event) {
            if(!event.equals(this.event)){
                assertEquals(event.getOdkial().getIndex(), odk);
                assertEquals(event.getKam().getIndex(), kam);
                System.out.println(nazov + " dostal tah");
                prijal++;
            }
        }

        @Override
        public void chybyTah(TahEvent event) {
        }

        @Override
        public void vyberFigurku(TahEvent event) {
            if(event.equals(this.event)){
                if(event.getTypFig()!= this.event.getTypFig()){
                    assertTrue(event.getTypFig()==typfig);
                    System.out.println(nazov + " dostal " + typfig);
                    prijal++;
                }
            }
        }

        @Override
        public void chyba(TahEvent event) {
           if(!event.equals(this.event)){
                fireChyba(event.getChyba());
            }
        }

        @Override
        public void sach(TahEvent event) {
            if(!event.equals(this.event)){
                fireSach(event.getFarba());
            }
        }

        @Override
        public void mat(TahEvent event) {
            if(!event.equals(this.event)){
                fireMat(event.getFarba());
            }
        }

        public void tah(Point a, Point b){
            fireTah(a, b, Figurka.Color.biela);
        }
        public void fFigurka(Point a, Point b, Figurka.Type typ){
            fireVyberFigurku(a, b, typ, Figurka.Color.biela);
        }
    };

    private void connect() throws NetworkGameException, NetworkTimeOutException, NetworkConnectionRefusedException, UnknownHostException, InterruptedException {
        
        class Vlakno implements Runnable {
            public void run() {
                try {
                    srv.connect("", 8190);
                } catch (Throwable ex) {
                    assertTrue(false);
                }
            }
        }
        clt.connect(InetAddress.getLocalHost().getHostAddress(), 8190);
        assertTrue(!srv.isConnected());
        assertTrue(!clt.isConnected());
        Thread vlk = new Thread(new Vlakno());
        vlk.start();
        Thread.sleep(1000);
        clt.connect(InetAddress.getLocalHost().getHostAddress(), 8190);
        assertTrue(srv.isConnected());
        assertTrue(clt.isConnected());
    }

    @Test
    public void sendRecive() throws NetworkGameException, NetworkTimeOutException, NetworkConnectionRefusedException, UnknownHostException, InterruptedException{
        connect();
        SachApp.gameStatus = new GameStatus(Figurka.Color.biela, null);
        final Handler srth = new Handler(0, 15, Type.kon, "Server");
        final Handler clth = new Handler(6, 24, Type.dama, "Client");
        srth.addTahListener(srv);
        clth.addTahListener(clt);
        srv.addTahListener(srth);
        clt.addTahListener(clth);
        
        class Vlakno1 implements Runnable {
            public void run() {

                    System.out.println("Server posiela figurku");
                    srth.fFigurka(new Point(6), new Point(24), Figurka.Type.dama);
                    while(srth.prijal!=1){
                        try {
                            Thread.sleep(100);
                        } catch (InterruptedException ex) {
                            Logger.getLogger(NetworkGameUnitTest.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    }
                    srth.fFigurka(new Point(5), new Point(24), null);
                    while(srth.prijal!=2){
                        try {
                            Thread.sleep(100);
                        } catch (InterruptedException ex) {
                            Logger.getLogger(NetworkGameUnitTest.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    }
                    System.out.println("Server posiela tah");
                    srth.tah(new Point(6), new Point(24));                    
                    while(srth.prijal!=3){
                        try {
                            Thread.sleep(100);
                        } catch (InterruptedException ex) {
                            Logger.getLogger(NetworkGameUnitTest.class.getName()).log(Level.SEVERE, null, ex);
                        }
                    }
            }
        }
        Thread vlk = new Thread(new Vlakno1());
        vlk.start();
        class Vlakno2 implements Runnable {
            public void run() {
                try {
                    System.out.println("Client caka");
                    while(clth.prijal!=1){
                        Thread.sleep(100);
                    }
                    clth.fFigurka(new Point(5), new Point(15), null);

                    System.out.println("Client posiela figurku");
                    clth.fFigurka(new Point(0), new Point(15), Figurka.Type.kon);
                    
                    while(clth.prijal!=3){
                        Thread.sleep(100);
                    }
                    System.out.println("Client posiela tah");
                    clth.tah(new Point(0), new Point(15));

                } catch (Throwable ex) {
                    assertTrue(false);
                }
            }
        }
        Thread vlk2 = new Thread(new Vlakno2());
        vlk2.start();

        while(clth.prijal != 3 || srth.prijal != 3){
            Thread.sleep(100);
        }
        
        assertEquals(srv.getHostName(), clt.getHostName());
        assertEquals(srv.getIpAddress(), clt.getIpAddress());
        Thread.sleep(1000);
        clt.closeConnection();
        srv.closeConnection();
        assertTrue(!srv.isConnected());
        assertTrue(!clt.isConnected());
    }

    @Test
    public void vynimky(){
        class Upr extends NetworkGame {

            @Override
            public boolean connect(String ip, int port) throws NetworkGameException, NetworkTimeOutException, NetworkConnectionRefusedException {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            @Override
            public boolean isConnected() {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            @Override
            public void closeConnection() throws NetworkGameException {
                throw new UnsupportedOperationException("Not supported yet.");
            }

            public void throwNetworkGameException(String p) throws NetworkGameException{
                if(p.equals("")){
                    throw new NetworkGameException();
                } else {
                    throw new NetworkGameException(p);
                }
            }
            public void throwNetworkTimeOutException() throws NetworkTimeOutException{
                throw new NetworkTimeOutException();
            }
            public void throwNetworkConnectionRefusedException() throws NetworkConnectionRefusedException{
                throw new NetworkConnectionRefusedException();
            }
        }

        Upr n = new Upr();
        try {
            n.throwNetworkGameException("");
        } catch (NetworkGameException ex) {
           assertEquals(ex.getMessage(), "Vyskytla sa necakana chyba v sietovej hre!");
        }
        try {
            n.throwNetworkGameException("text");
        } catch (NetworkGameException ex) {
            assertEquals(ex.getMessage(), "Vyskytla sa necakana chyba v sietovej hre: text");
        }
        try {
            n.throwNetworkConnectionRefusedException();
        } catch (NetworkConnectionRefusedException ex) {
            assertEquals(ex.getMessage(), "Spojenie bolo neocakavane prerusene!");
        }
        try {
            n.throwNetworkTimeOutException();
        } catch (NetworkTimeOutException ex) {
            assertEquals(ex.getMessage(), "Casovy limit spojenia vyprsal!");
        }
    }
}
